/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Thanks to abaddon for proof-reading our comm.c and pointing out bugs.  *
 *  Any remaining bugs are, of course, our work, not his.  :)              *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/*
 * This file contains all of the OS-dependent stuff:
 *   startup, signals, BSD sockets for tcp/ip, i/o, timing.
 *
 * The data flow for input is:
 *    Game_loop ---> Read_from_descriptor ---> Read
 *    Game_loop ---> Read_from_buffer
 *
 * The data flow for output is:
 *    Game_loop ---> Process_Output ---> Write_to_descriptor -> Write
 *
 * The OS-dependent functions are Read_from_descriptor and Write_to_descriptor.
 * -- Furey  26 Jan 1993
 */

#if defined(macintosh)
#include <types.h>
#include <sys/errno.h>
#include <GUSI.H>
#elif defined(WIN32)
#include <winsock.h>
#include <sys/timeb.h>
#else
#include <sys/types.h>
//#include <sys/time.h>
#include <errno.h>
#if defined(WIN32)
#include <sys/timeb.h>
#endif
#endif

#if defined(WIN32)
#include <time.h>
#else
#include <sys/time.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <stdarg.h>

#include "merc.h"
#include "crash.h"
#include "color.h"
#include "recycle.h"

/* command procedures needed */
DECLARE_DO_FUN(do_help          );
DECLARE_DO_FUN(do_look          );
DECLARE_DO_FUN(do_skills        );
DECLARE_DO_FUN(do_outfit        );
DECLARE_DO_FUN(do_unread        );


#include <signal.h>

/*
 * Malloc debugging stuff.
 */
#if defined(sun)
#undef MALLOC_DEBUG
#endif

#if defined(MALLOC_DEBUG)
#include <malloc.h>
extern  int     malloc_debug    args( ( int  ) );
extern  int     malloc_verify   args( ( void ) );
#endif



/*
 * Signal handling.
 * Apollo has a problem with __attribute(atomic) in signal.h,
 *   I dance around it.
 */
#if defined(apollo)
#define __attribute(x)
#endif

#if defined(unix)
#include <signal.h>
#endif

#if defined(apollo)
#undef __attribute
#endif



/*
 * Socket and TCP/IP stuff.
 */
#if     defined(macintosh) || defined(MSDOS)
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include "telnet.h"
const   char    echo_off_str    [] = { '\0' };
const   char    echo_on_str     [] = { '\0' };
const   char    go_ahead_str    [] = { '\0' };
#endif

#if defined(WIN32)
#include "telnet.h"
const   char    echo_off_str    [] = { IAC, WILL, TELOPT_ECHO, '\0' };
const   char    echo_on_str     [] = { IAC, WONT, TELOPT_ECHO, '\0' };
const   char    go_ahead_str    [] = { IAC, GA, '\0' };
#endif

#if     defined(unix)
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include "telnet.h"
#include <signal.h>
#if !defined(STDOUT_FILENO)
#define STDOUT_FILENO 1
#endif
const   char    echo_off_str    [] = { IAC, WILL, TELOPT_ECHO, '\0' };
const   char    echo_on_str     [] = { IAC, WONT, TELOPT_ECHO, '\0' };
const   char    go_ahead_str    [] = { IAC, GA, '\0' };
#endif



/*
 * OS-dependent declarations.
 */
#if     defined(_AIX)
#include <sys/select.h>
int     accept          args( ( int s, struct sockaddr *addr, int *addrlen ) );
int     bind            args( ( int s, struct sockaddr *name, int namelen ) );
void    bzero           args( ( char *b, int length ) );
int     getpeername     args( ( int s, struct sockaddr *name, int *namelen ) );
int     getsockname     args( ( int s, struct sockaddr *name, int *namelen ) );
int     gettimeofday    args( ( struct timeval *tp, struct timezone *tzp ) );
int     listen          args( ( int s, int backlog ) );
int     setsockopt      args( ( int s, int level, int optname, void *optval,
			    int optlen ) );
int     socket          args( ( int domain, int type, int protocol ) );
#endif

#if     defined(apollo)
#include <unistd.h>
void    bzero           args( ( char *b, int length ) );
#endif

#if     defined(__hpux)
int     accept          args( ( int s, void *addr, int *addrlen ) );
int     bind            args( ( int s, const void *addr, int addrlen ) );
void    bzero           args( ( char *b, int length ) );
int     getpeername     args( ( int s, void *addr, int *addrlen ) );
int     getsockname     args( ( int s, void *name, int *addrlen ) );
int     gettimeofday    args( ( struct timeval *tp, struct timezone *tzp ) );
int     listen          args( ( int s, int backlog ) );
int     setsockopt      args( ( int s, int level, int optname,
				const void *optval, int optlen ) );
int     socket          args( ( int domain, int type, int protocol ) );
#endif

#if     defined(interactive)
#include <net/errno.h>
#include <sys/fnctl.h>
#endif

#if     defined(linux)
int     close           args( ( int fd ) );
//int     getpeername     args( ( int s, struct sockaddr *name, socklen_t *namelen ) );
//int     getsockname     args( ( int s, struct sockaddr *name, socklen_t *namelen ) );
int     gettimeofday    args( ( struct timeval *tp, struct timezone *tzp ) );
//int     listen          args( ( int s, unsigned int backlog ) );
int     read            args( ( int fd, char *buf, int nbyte ) );
int     select          args( ( int width, fd_set *readfds, fd_set *writefds,
			    fd_set *exceptfds, struct timeval *timeout ) );
int     socket          args( ( int domain, int type, int protocol ) );
int     write           args( ( int fd, char *buf, int nbyte ) );

pid_t   waitpid         args( ( pid_t pid, int *status, int options ) );
pid_t   fork            args( ( void ) );
int     kill            args( ( pid_t pid, int sig ) );
int     pipe            args( ( int filedes[2] ) );
int     dup2            args( ( int oldfd, int newfd ) );
int     execl           args( ( const char *path, const char *arg, ... ) );
#endif

#if     defined(macintosh)
#include <console.h>
#include <fcntl.h>
#include <unix.h>
///struct  timeval
///{
///	time_t  tv_sec;
///	time_t  tv_usec;
///};
#if     !defined(isascii)
#define isascii(c)              ( (c) < 0200 )
#endif
static  long                    theKeys [4];

///int     gettimeofday            args( ( struct timeval *tp, void *tzp ) );
#endif

#if     defined(MIPS_OS)
extern  int             errno;
#endif

#if     defined(MSDOS)
int     gettimeofday    args( ( struct timeval *tp, void *tzp ) );
int     kbhit           args( ( void ) );
#endif

#if     defined(WIN32)
#define EINTR           WSAEINTR
void gettimeofday       args( ( struct timeval *tp, void *tzp ) );
#endif

#if     defined(NeXT)
int     close           args( ( int fd ) );
int     fcntl           args( ( int fd, int cmd, int arg ) );
#if     !defined(htons)
u_short htons           args( ( u_short hostshort ) );
#endif
#if     !defined(ntohl)
u_long  ntohl           args( ( u_long hostlong ) );
#endif
int     read            args( ( int fd, char *buf, int nbyte ) );
int     select          args( ( int width, fd_set *readfds, fd_set *writefds,
			    fd_set *exceptfds, struct timeval *timeout ) );
int     write           args( ( int fd, char *buf, int nbyte ) );
#endif

#if     defined(sequent)
int     accept          args( ( int s, struct sockaddr *addr, int *addrlen ) );
int     bind            args( ( int s, struct sockaddr *name, int namelen ) );
int     close           args( ( int fd ) );
int     fcntl           args( ( int fd, int cmd, int arg ) );
int     getpeername     args( ( int s, struct sockaddr *name, int *namelen ) );
int     getsockname     args( ( int s, struct sockaddr *name, int *namelen ) );
int     gettimeofday    args( ( struct timeval *tp, struct timezone *tzp ) );
#if     !defined(htons)
u_short htons           args( ( u_short hostshort ) );
#endif
int     listen          args( ( int s, int backlog ) );
#if     !defined(ntohl)
u_long  ntohl           args( ( u_long hostlong ) );
#endif
int     read            args( ( int fd, char *buf, int nbyte ) );
int     select          args( ( int width, fd_set *readfds, fd_set *writefds,
			    fd_set *exceptfds, struct timeval *timeout ) );
int     setsockopt      args( ( int s, int level, int optname, caddr_t optval,
			    int optlen ) );
int     socket          args( ( int domain, int type, int protocol ) );
int     write           args( ( int fd, char *buf, int nbyte ) );
#endif

/* This includes Solaris Sys V as well */
#if defined(sun)
int     accept          args( ( int s, struct sockaddr *addr, int *addrlen ) );
int     bind            args( ( int s, struct sockaddr *name, int namelen ) );
void    bzero           args( ( char *b, int length ) );
int     close           args( ( int fd ) );
int     getpeername     args( ( int s, struct sockaddr *name, int *namelen ) );
int     getsockname     args( ( int s, struct sockaddr *name, int *namelen ) );
int     gettimeofday    args( ( struct timeval *tp, struct timezone *tzp ) );
int     listen          args( ( int s, int backlog ) );
int     read            args( ( int fd, char *buf, int nbyte ) );
int     select          args( ( int width, fd_set *readfds, fd_set *writefds,
			    fd_set *exceptfds, struct timeval *timeout ) );
#if defined(SYSV)
int setsockopt          args( ( int s, int level, int optname,
			    const char *optval, int optlen ) );
#else
int     setsockopt      args( ( int s, int level, int optname, void *optval,
			    int optlen ) );
#endif
int     socket          args( ( int domain, int type, int protocol ) );
int     write           args( ( int fd, char *buf, int nbyte ) );
#endif

#if defined(ultrix)
int     accept          args( ( int s, struct sockaddr *addr, int *addrlen ) );
int     bind            args( ( int s, struct sockaddr *name, int namelen ) );
void    bzero           args( ( char *b, int length ) );
int     close           args( ( int fd ) );
int     getpeername     args( ( int s, struct sockaddr *name, int *namelen ) );
int     getsockname     args( ( int s, struct sockaddr *name, int *namelen ) );
int     gettimeofday    args( ( struct timeval *tp, struct timezone *tzp ) );
int     listen          args( ( int s, int backlog ) );
int     read            args( ( int fd, char *buf, int nbyte ) );
int     select          args( ( int width, fd_set *readfds, fd_set *writefds,
			    fd_set *exceptfds, struct timeval *timeout ) );
int     setsockopt      args( ( int s, int level, int optname, void *optval,
			    int optlen ) );
int     socket          args( ( int domain, int type, int protocol ) );
int     write           args( ( int fd, char *buf, int nbyte ) );
#endif



/*
 * Global variables.
 */
DESCRIPTOR_DATA *   descriptor_list;    /* All open descriptors         */
DESCRIPTOR_DATA *   d_next;             /* Next descriptor in loop      */
FILE *              fpReserve;          /* Reserved file handle         */
bool                god;                /* All new chars are gods!      */
bool                merc_down;          /* Shutdown                     */
int                 wizlock;            /* Game is wizlocked            */
char                str_boot_time[MAX_INPUT_LENGTH];
char                str_reboot_time[MAX_INPUT_LENGTH];
char                str_warmboot_time[MAX_INPUT_LENGTH];
time_t              current_time;       /* time of this pulse           */
time_t              boot_time;
time_t              reboot_time;
time_t              warmboot_time;
int                 max_on = 0;
int                 mudport;            /* mudport                      */
bool                MOBtrigger = TRUE;  /* act() switch                 */
bool                reboot_set = FALSE;

char                    *invalid_name_parts;    /* list of name parts that are illegal */


/*
 * OS-dependent local functions.
 */
#if defined(macintosh)
void    game_loop_unix          args( ( int control ) );
//void    game_loop_mac_msdos     args( ( void ) );
int     init_socket             args( ( int port ) );
void    init_descriptor         args( ( int control ) );
bool    read_from_descriptor    args( ( DESCRIPTOR_DATA *d ) );
bool    write_to_descriptor     args( ( int desc, char *txt, int length ) );
#endif

#if defined(MSDOS)
void    game_loop_mac_msdos     args( ( void ) );
bool    read_from_descriptor    args( ( DESCRIPTOR_DATA *d ) );
bool    write_to_descriptor     args( ( int desc, char *txt, int length ) );
#endif

#if defined(unix) || defined(WIN32)
void    game_loop_unix          args( ( int control ) );
int     init_socket             args( ( int port ) );
void    init_descriptor         args( ( int control ) );
bool    read_from_descriptor    args( ( DESCRIPTOR_DATA *d ) );
bool    write_to_descriptor     args( ( int desc, char *txt, int length ) );
#endif




/*
 * Other local functions (OS-independent).
 */
void    load_invalid_name_parts args( ( void ) );
int     main                    args( ( int argc, char **argv ) );
void    nanny                   args( ( DESCRIPTOR_DATA *d, char *argument ) );
bool    process_output          args( ( DESCRIPTOR_DATA *d, bool fPrompt ) );
void    read_from_buffer        args( ( DESCRIPTOR_DATA *d ) );
void    stop_idling             args( ( CHAR_DATA *ch ) );
void    bust_a_prompt           args( ( CHAR_DATA *ch ) );
void    generate_stats          args( ( DESCRIPTOR_DATA *d ) );
void	pick_hometown		args( ( DESCRIPTOR_DATA *d ) );
void    my_sig_handler          args( ( int signum ) );

int port, control;

int main( int argc, char **argv )
{
    struct timeval now_time;
    bool fWarmBoot = FALSE;

    /*
     * Memory debugging if needed.
     */
#if defined(MALLOC_DEBUG)
    malloc_debug( 2 );
#endif

	 /*
	  * Macintosh sockets initialization.
	  */
#if defined(macintosh)
 	GUSISetup(GUSIwithSIOUXSockets);
	GUSISetup(GUSIwithInternetSockets);
	GUSISetup(GUSIwithUnixSockets);
#endif
	

    /*
     * Init time.
     */
    gettimeofday( &now_time, NULL );
    current_time        = (time_t) now_time.tv_sec;
    strcpy( str_boot_time, ctime( &current_time ) );
    *(str_boot_time + strlen(str_boot_time) - 1) = '\0';
    boot_time = time(0);
    strcpy( str_warmboot_time, ctime( &current_time ) );
    *(str_warmboot_time + strlen(str_warmboot_time) - 1) = '\0';
    warmboot_time = time(0);
    reboot_time = -1;
    sprintf(str_reboot_time, "none");

    /*
     * Macintosh console initialization.
     */
///#if defined(macintosh)
///    console_options.nrows = 31;
///    cshow( stdout );
///    csetmode( C_RAW, stdin );
///    cecho2file( "log file", 1, stderr );
///#endif

    /*
     * Reserve one channel for our use.
     */
    if ( ( fpReserve = fopen( NULL_FILE, "r" ) ) == NULL )
    {
	perror( NULL_FILE );
	exit( 1 );
    }

    /*
     * Get the port number.
     */
    port = 4000;
    if ( argc > 1 )
    {
	if ( !is_number( argv[1] ) )
	{
	    fprintf( stderr, "Usage: %s [port #]\n", argv[0] );
	    exit( 1 );
	}
	else if ( ( port = atoi( argv[1] ) ) <= 1024 )
	{
	    fprintf( stderr, "Port number must be above 1024.\n" );
	    exit( 1 );
	}

	if (argv[2] && argv[2][0])
 	{
 		fWarmBoot = TRUE;
 		control = atoi(argv[3]);
 	}
 	else
 		fWarmBoot = FALSE;
     }
 
    /*
     * Run the game.
     */
#if defined(macintosh)
    control = init_socket( port );
    boot_db();
    sprintf( log_buf, "ROM is ready to rock on port %d.", port );
    log_string( log_buf );
    game_loop_unix( control );
    //game_loop_mac_msdos( );
    close (control);
#endif
     
#if defined(MSDOS)
    boot_db();

    load_invalid_name_parts( );

    log_string( "ROM is ready to rock." );
    game_loop_mac_msdos( );
#endif

#if defined(unix) || defined(WIN32)
    crash_install_handlers( );
    mudport=port;

    if (!fWarmBoot)
	control = init_socket( port );
	    
    boot_db();
    load_invalid_name_parts( );
    sprintf( log_buf, "ROM is ready to rock on port %d.", port );
    log_string( log_buf );
    
    if (fWarmBoot)
    	warmboot_recover();
    
    game_loop_unix( control );
    #if defined(WIN32)
    closesocket( control );
    WSACleanup();
    #else
    close (control);
    #endif
#endif

    /*
     * That's all, folks.
     */
    log_string( "Normal termination of game." );
    exit( 0 );
    return 0;
}


#if defined(unix) || defined(macintosh) || defined(WIN32)
int init_socket( int port )
{
    static struct sockaddr_in sa_zero;
    struct sockaddr_in sa;
    int x = 1;
    int fd;

#if defined(WIN32)
    WORD version = MAKEWORD( 1, 1 );
    WSADATA wsaData;
    
    int error = WSAStartup( version, &wsaData );
    if ( error )
    {
        perror( "Init_socket: WINSOCK.DLL unavailable." );
        exit( 1 );
    }
    
    if ( (fd = socket(PF_INET, SOCK_STREAM, 0)) < 0 )
    {
        perror( "Init_socket: unable to open socket." );
        exit( 1 );
    }
#else
    if ( ( fd = socket( AF_INET, SOCK_STREAM, 0 ) ) < 0 )
    {
	perror( "Init_socket: socket" );
	exit( 1 );
    }
#endif

#if !defined(macintosh)
    if ( setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *) &x, sizeof(x)) < 0 )
    {
	perror( "Init_socket: SO_REUSEADDR" );
	#if defined(WIN32)
	closesocket( fd );
	#else
	close(fd);
	#endif
	exit( 1 );
    }
#endif

#if defined(SO_DONTLINGER) && !defined(SYSV)
    {
	struct  linger  ld;

	ld.l_onoff  = 1;
	ld.l_linger = 1000;

	if ( setsockopt( fd, SOL_SOCKET, SO_DONTLINGER,
	(char *) &ld, sizeof(ld) ) < 0 )
	{
	    perror( "Init_socket: SO_DONTLINGER" );
	    #if defined(WIN32)
	    closesocket( fd );
	    #else
	    close(fd);
	    #endif
	    exit( 1 );
	}
    }
#endif

    sa              = sa_zero;
    #if defined(WIN32)
    sa.sin_family   = PF_INET;
    #else
    sa.sin_family   = AF_INET;
    #endif
    sa.sin_port     = htons( port );

    if ( bind( fd, (struct sockaddr *) &sa, sizeof(sa) ) < 0 )
    {
	perror("Init socket: bind" );
	#if defined(WIN32)
	closesocket( fd );
	#else
	close(fd);
	#endif
	exit(1);
    }


    if ( listen( fd, 3 ) < 0 )
    {
	perror("Init socket: listen");
	#if defined(WIN32)
	closesocket( fd );
	#else
	close(fd);
	#endif
	exit(1);
    }

    return fd;
}
#endif



#if defined(MSDOS)
void game_loop_mac_msdos( void )
{
    struct timeval last_time;
    struct timeval now_time;
    static DESCRIPTOR_DATA dcon;

    gettimeofday( &last_time, NULL );
    current_time = (time_t) last_time.tv_sec;

    /*
     * New_descriptor analogue.
     */
    dcon.descriptor     = 0;
    dcon.connected      = CON_GET_NAME;
    dcon.host           = str_dup( "localhost" );
    dcon.outsize        = 2000;
    dcon.outbuf         = alloc_mem( dcon.outsize );
    dcon.next           = descriptor_list;
    dcon.showstr_head   = NULL;
    dcon.showstr_point  = NULL;
    descriptor_list     = &dcon;

    /*
     * Send the greeting.
     */
    {
	extern char * help_greeting;
	if ( help_greeting[0] == '.' )
	    write_to_buffer( &dcon, help_greeting+1, 0 );
	else
	    write_to_buffer( &dcon, help_greeting  , 0 );
    }

    /* Main loop */
    while ( !merc_down )
    {
	DESCRIPTOR_DATA *d;

	/*
	 * Process input.
	 */
	for ( d = descriptor_list; d != NULL; d = d_next )
	{
	    d_next      = d->next;
	    d->fcommand = FALSE;

#if defined(MSDOS)
	    if ( kbhit( ) )
#endif
	    {
		if ( d->character != NULL )
		    d->character->timer = 0;
		if ( !read_from_descriptor( d ) )
		{
		    if ( d->character != NULL)
			save_char_obj( d->character, TRUE );
		    d->outtop   = 0;
		    close_socket( d, TRUE );
		    continue;
		}
	    }

	    if (d->character != NULL && d->character->daze > 0)
		--d->character->daze;

	    if ( d->character != NULL && d->character->wait > 0 )
	    {
		--d->character->wait;
		continue;
	    }

	    read_from_buffer( d );
	    if ( d->incomm[0] != '\0' )
	    {
		d->fcommand     = TRUE;
		stop_idling( d->character );

		if ( d->connected == CON_PLAYING )
		    substitute_alias( d, d->incomm );
		else
		    nanny( d, d->incomm );

		d->incomm[0]    = '\0';
	    }
	}



	/*
	 * Autonomous game motion.
	 */
	update_handler( );



	/*
	 * Output.
	 */
	for ( d = descriptor_list; d != NULL; d = d_next )
	{
	    d_next = d->next;

	    if ( ( d->fcommand || d->outtop > 0 ) )
	    {
		if ( !process_output( d, TRUE ) )
		{
		    if ( d->character != NULL && d->character->level > 1)
			save_char_obj( d->character, TRUE );
		    d->outtop   = 0;
		    close_socket( d, TRUE );
		}
	    }
	}



	/*
	 * Synchronize to a clock.
	 * Busy wait (blargh).
	 */
	now_time = last_time;
	for ( ; ; )
	{
	    int delta;

#if defined(MSDOS)
	    if ( kbhit( ) )
#endif
	    {
		if ( dcon.character != NULL )
		    dcon.character->timer = 0;
		if ( !read_from_descriptor( &dcon ) )
		{
		    if ( dcon.character != NULL && d->character->level > 1)
			save_char_obj( d->character, TRUE );
		    dcon.outtop = 0;
		    close_socket( &dcon, TRUE );
		}
#if defined(MSDOS)
		break;
#endif
	    }

	    gettimeofday( &now_time, NULL );
	    delta = ( now_time.tv_sec  - last_time.tv_sec  ) * 1000 * 1000
		  + ( now_time.tv_usec - last_time.tv_usec );
	    if ( delta >= 1000000 / PULSE_PER_SECOND )
		break;
	}
	last_time    = now_time;
	current_time = (time_t) last_time.tv_sec;
    }

    return;
}
#endif


/* 
 * Here comes the ident driver code.
 * - Wreck
 */
#if defined(unix)

/*
 * Almost the same as read_from_buffer...
 */
bool read_from_ident( int fd, char *buffer )
{
    static char inbuf[MAX_STRING_LENGTH*2];
    int iStart, i, j, k;

    /* Check for overflow. */
    iStart = strlen( inbuf );
    if ( iStart >= sizeof( inbuf ) - 10 )
    {
	log_string( "Ident input overflow!!!" );
	return FALSE;
    }

    /* Snarf input. */
    for ( ; ; )
    {
	int nRead;

	nRead = read( fd, inbuf + iStart, sizeof( inbuf ) - 10 - iStart );
	if ( nRead > 0 )
	{
	    iStart += nRead;
	    if ( inbuf[iStart-2] == '\n' || inbuf[iStart-2] == '\r' )
		break;
	}
	else if ( nRead == 0 )
	{
	    return FALSE;
	}
	else if ( errno == EWOULDBLOCK )
	    break;
	else
	{
	    perror( "Read_from_ident" );
	    return FALSE;
	}
    }

    inbuf[iStart] = '\0';

    /*
     * Look for at least one new line.
     */
    for ( i = 0; inbuf[i] != '\n' && inbuf[i] != '\r'; i++ )
    {
	if ( inbuf[i] == '\0' )
	    return FALSE;
    }

    /*
     * Canonical input processing.
     */
    for ( i = 0, k = 0; inbuf[i] != '\n' && inbuf[i] != '\r'; i++ )
    {
	if ( inbuf[i] == '\b' && k > 0 )
	    --k;
	else if ( isascii( inbuf[i] ) && isprint( inbuf[i] ) )
	    buffer[k++] = inbuf[i];
    }

    /*
     * Finish off the line.
     */
    if ( k == 0 )
	buffer[k++] = ' ';
    buffer[k] = '\0';

    /*
     * Shift the input buffer.
     */
    while ( inbuf[i] == '\n' || inbuf[i] == '\r' )
	i++;
    for ( j = 0; ( inbuf[j] = inbuf[i+j] ) != '\0'; j++ )
	;
    
    return TRUE;
}

/*
 * Process input that we got from the ident process.
 */
void process_ident( DESCRIPTOR_DATA *d )
{
    char buffer[MAX_INPUT_LENGTH];
    char address[MAX_INPUT_LENGTH];
    CHAR_DATA *ch=CH( d );
    char *user;
    sh_int results=0;
    int status;
    
    buffer[0]='\0';
    
    if ( !read_from_ident( d->ifd, buffer ) || IS_NULLSTR( buffer ) )
	return;
    
    /* using first arg since we want to keep case */
    user=first_arg( buffer, address, FALSE );
    
    /* replace and set some states */
    if ( !IS_NULLSTR( user ) )
    {
	free_string( d->ident );
	d->ident = str_dup( user );
	SET_BIT( results, 2 );
    }
    if ( !IS_NULLSTR( address ) )
    {
	free_string( d->host );
	d->host = str_dup( address );
	SET_BIT( results, 1 );
    }
    
    /* do sensible output */
    if ( results==1 ) /* address only */
    {
	/*
	 * Change the two lines below to your notification function...
	 * (wiznet, ..., whatever)
	 */
	sprintf( log_buf, "%s has address %s.", ch->name, address );
	log_string( log_buf );
    }
    else if ( results==2 || results==3 ) /* ident only, or both */
    {
	/*
	 * Change the two lines below to your notification function...
	 * (wiznet, ..., whatever)
	 */
	sprintf( log_buf, "%s is %s@%s.", ch->name, user, address );
	log_string( log_buf );
    }
    else
    {
	sprintf( log_buf, "%s could not be identified.", ch->name );
	log_string( log_buf );
    }
    
    /* close descriptor and kill ident process */
    close( d->ifd );
    d->ifd=-1;
    /* 
     * we don't have to check here, 
     * cos the child is probably dead already. (but out of safety we do)
     * 
     * (later) I found this not to be true. The call to waitpid( ) is
     * necessary, because otherwise the child processes become zombie
     * and keep lingering around... The waitpid( ) removes them.
     */
    waitpid( d->ipid, &status, WNOHANG );
    d->ipid=-1;
    
    return;    
}


void create_ident( DESCRIPTOR_DATA *d, long ip, sh_int port )
{
    int fds[2];
    pid_t pid;
    
    /* create pipe first */
    if ( pipe( fds )!=0 )
    {
	perror( "Create_ident: pipe: " );
	return;
    }
    
    if ( (pid=fork( ))>0 )
    {
	/* parent process */
	d->ifd=fds[0];
	close( fds[1] );
	d->ipid=pid;
    }
    else if ( pid==0 )
    {
	/* child process */
	char str_ip[64], str_local[64], str_remote[64];

	if ( dup2(fds[1], STDOUT_FILENO) != STDOUT_FILENO)
	{
	    perror( "Create_ident: dup2(stdout): " );
	    return;
	}
	
	sprintf( str_local, "%d", mudport );
	sprintf( str_remote, "%d", port );
	sprintf( str_ip, "%ld", ip );
	execl( "../src/resolve", "resolve", str_local, str_ip, str_remote, 0 );
	/* Still here --> hmm. An error. */
	log_string( "Exec failed; Closing child." );
	exit( 0 );
    }
    else 
    {
	/* error */
	perror( "Create_ident: fork" );
	close( fds[0] );
	close( fds[1] );
    }
}

#endif


#if defined(unix) || defined(macintosh) || defined(WIN32)
void game_loop_unix( int control )
{
    static struct timeval null_time;
    struct timeval last_time;

#if defined(unix)    
    struct sigaction sa;
#endif

#if !defined(WIN32)
    signal( SIGPIPE, SIG_IGN );
#endif

#if defined(unix)
    sa.sa_handler = my_sig_handler;
    sigemptyset( &sa.sa_mask );
//    sa.sa_mask    = 0;
    sa.sa_flags   = SA_RESTART;
    if ( sigaction(SIGCLD, &sa, 0) < 0 )
    {
	perror( "Failed to install SIGCLD handler\n" );
	exit( 0 );
    }
#endif

    gettimeofday( &last_time, NULL );
    current_time = (time_t) last_time.tv_sec;

    /* Main loop */
    while ( !merc_down )
    {
	fd_set in_set;
	fd_set out_set;
	fd_set exc_set;
	DESCRIPTOR_DATA *d;
	int maxdesc;

#if defined(MALLOC_DEBUG)
	if ( malloc_verify( ) != 1 )
	    abort( );
#endif

	/*
	 * Poll all active descriptors.
	 */
	FD_ZERO( &in_set  );
	FD_ZERO( &out_set );
	FD_ZERO( &exc_set );
	FD_SET( control, &in_set );
	maxdesc = control;
	for ( d = descriptor_list; d; d = d->next )
	{
	    maxdesc = UMAX( maxdesc, d->descriptor );
	    FD_SET( d->descriptor, &in_set  );
	    FD_SET( d->descriptor, &out_set );
	    FD_SET( d->descriptor, &exc_set );
#if defined(unix)
	    if ( d->ifd!=-1 && d->ipid!=-1 )
	    {
		maxdesc = UMAX( maxdesc, d->ifd );
		FD_SET( d->ifd, &in_set );
	    }
#endif
	}

#if defined(macintosh)
	if ( select( maxdesc+1, &in_set, &out_set, &exc_set, &null_time ) < 0 )
	{
	    perror( "Game_loop: select: poll" );
	    exit( 1 );
	}
#else
	while ( select( maxdesc+1, &in_set, &out_set, &exc_set, &null_time ) < 0 )
	{
	    if ( errno == EINTR )
		continue;
	    perror( "Game_loop: select: poll" );
	    exit( 1 );
	}
#endif

	/*
	 * New connection?
	 */
	if ( FD_ISSET( control, &in_set ) )
	    init_descriptor( control );

	/*
	 * Kick out the freaky folks.
	 */
	for ( d = descriptor_list; d != NULL; d = d_next )
	{
	    d_next = d->next;   
	    if ( FD_ISSET( d->descriptor, &exc_set ) )
	    {
		FD_CLR( d->descriptor, &in_set  );
		FD_CLR( d->descriptor, &out_set );
		if ( d->character && d->character->level > 1 )
		{
		    save_char_obj( d->character, TRUE );
		    if ( d->connected != CON_PLAYING )
			nuke_pets( d->character );
		}
		
		d->outtop = 0;
		close_socket( d, TRUE );
	    }
	}

	/*
	 * Process input.
	 */
	for ( d = descriptor_list; d != NULL; d = d_next )
	{
	    d_next      = d->next;
	    d->fcommand = FALSE;

	    if ( FD_ISSET( d->descriptor, &in_set ) )
	    {
		if ( d->character != NULL )
		    d->character->timer = 0;
		if ( !read_from_descriptor( d ) )
		{
		    FD_CLR( d->descriptor, &out_set );
		    if ( d->character != NULL && d->character->level > 1 )
		    {
			save_char_obj( d->character, TRUE );
			if ( d->connected != CON_PLAYING )
			    nuke_pets( d->character );
		    }

		    d->outtop = 0;
		    close_socket( d, TRUE );
		    continue;
		}
	    }

#if defined(unix)
	    /* check for input from the ident */
	    if ( ( d->connected==CON_PLAYING || CH(d)!=NULL ) && 
		 d->ifd!=-1 && FD_ISSET( d->ifd, &in_set ) )
		process_ident( d );
#endif

	    if (d->character != NULL && d->character->daze > 0)
		--d->character->daze;

	    if ( d->character != NULL && d->character->wait > 0 )
	    {
		--d->character->wait;
		continue;
	    }

	    read_from_buffer( d );
	    if ( d->incomm[0] != '\0' )
	    {
		d->fcommand     = TRUE;
		stop_idling( d->character );

		if (d->showstr_point)
		    show_string(d,d->incomm);
		else if ( d->connected == CON_PLAYING )
		    substitute_alias( d, d->incomm );
		else
		    nanny( d, d->incomm );

		d->incomm[0]    = '\0';
	    }
	}



	/*
	 * Autonomous game motion.
	 */
	update_handler( );



	/*
	 * Output.
	 */
	for ( d = descriptor_list; d != NULL; d = d_next )
	{
	    d_next = d->next;

	    if ( ( d->fcommand || d->outtop > 0 )
	    &&   FD_ISSET(d->descriptor, &out_set) )
	    {
		if ( !process_output( d, TRUE ) )
		{
		    if ( d->character != NULL && d->character->level > 1 )
		    {
			save_char_obj( d->character, TRUE );
			if ( d->connected != CON_PLAYING )
			    nuke_pets( d->character );
		    }

		    d->outtop = 0;
		    close_socket( d, TRUE );
		}
	    }
	}



	/*
	 * Synchronize to a clock.
	 * Sleep( last_time + 1/PULSE_PER_SECOND - now ).
	 * Careful here of signed versus unsigned arithmetic.
	 */
	{
	    struct timeval now_time;
	    long secDelta;
	    long usecDelta;

	    gettimeofday( &now_time, NULL );
	    usecDelta   = ((int) last_time.tv_usec) - ((int) now_time.tv_usec)
			+ 1000000 / PULSE_PER_SECOND;
	    secDelta    = ((int) last_time.tv_sec ) - ((int) now_time.tv_sec );
	    while ( usecDelta < 0 )
	    {
		usecDelta += 1000000;
		secDelta  -= 1;
	    }

	    while ( usecDelta >= 1000000 )
	    {
		usecDelta -= 1000000;
		secDelta  += 1;
	    }

	    if ( secDelta > 0 || ( secDelta == 0 && usecDelta > 0 ) )
	    {
		struct timeval stall_time;

		stall_time.tv_usec = usecDelta;
		stall_time.tv_sec  = secDelta;

#if defined(macintosh)
		if ( select( 0, NULL, NULL, NULL, &stall_time ) < 0 )
		{
		    perror( "Game_loop: select: stall" );
		    exit( 1 );
		}
#elif defined(WIN32)
        Sleep( stall_time.tv_sec * 1000 + stall_time.tv_usec / 1000 );
#else
		while ( select( 0, NULL, NULL, NULL, &stall_time ) < 0 )
		{
		    if ( errno == EINTR )
			continue;
		    perror( "Game_loop: select: stall" );
		    exit( 1 );
		}
#endif
	    }
	}

	gettimeofday( &last_time, NULL );
	current_time = (time_t) last_time.tv_sec;
    }

    return;
}
#endif



#if defined(unix) || defined(macintosh) || defined(WIN32)
void init_descriptor( int control )
{
    char buf[MAX_STRING_LENGTH];
    DESCRIPTOR_DATA *dnew;
    struct sockaddr_in sock;
    struct hostent *from;
    int desc;
    int size;

    size = sizeof(sock);
    getsockname( control, (struct sockaddr *) &sock, &size );
    if ( ( desc = accept( control, (struct sockaddr *) &sock, &size) ) < 0 )
    {
	perror( "New_descriptor: accept" );
	return;
    }
#if defined(WIN32)
    {
        unsigned long turnOn = 1;
        if ( ioctlsocket(desc, FIONBIO, &turnOn) != 0 )
        {
            bug( "New_descriptor: ioctlsocket failed with error %d.", WSAGetLastError() );
            return;
        }
    }
#else
    #if !defined(FNDELAY)
    #define FNDELAY O_NDELAY
    #endif

    if ( fcntl( desc, F_SETFL, FNDELAY ) == -1 )
    {
	    perror( "New_descriptor: fcntl: FNDELAY" );
	    return;
    }
#endif

    /*
     * Cons a new descriptor.
     */
    dnew = new_descriptor(); /* new_descriptor now also allocates things */
    dnew->descriptor = desc;

    size = sizeof(sock);
    if ( getpeername( desc, (struct sockaddr *) &sock, &size ) < 0 )
    {
	perror( "New_descriptor: getpeername" );
	dnew->host = str_dup( "(unknown)" );
    }
    else
    {
	/*
	 * Would be nice to use inet_ntoa here but it takes a struct arg,
	 * which ain't very compatible between gcc and system libraries.
	 */
	int addr;

#if defined(unix)
	create_ident( dnew, sock.sin_addr.s_addr, ntohs( sock.sin_port ) );
#endif
	addr = ntohl( sock.sin_addr.s_addr );
	sprintf( buf, "%d.%d.%d.%d",
	    ( addr >> 24 ) & 0xFF, ( addr >> 16 ) & 0xFF,
	    ( addr >>  8 ) & 0xFF, ( addr       ) & 0xFF
	    );
	/* Use just IP address for now. */
	from = gethostbyaddr( (char *) &sock.sin_addr,
	    sizeof(sock.sin_addr), AF_INET );
	dnew->host = str_dup( from ? from->h_name : buf );
/*      dnew->host=str_dup( buf );*/
#if defined(unix)
	dnew->port=ntohs( sock.sin_port );
#endif
	sprintf( log_buf, "New socket %d from %s", dnew->descriptor, buf );
	log_string( log_buf );
/*
 *      addr = ntohl( sock.sin_addr.s_addr );
 *      sprintf( buf, "%d.%d.%d.%d",
 *          ( addr >> 24 ) & 0xFF, ( addr >> 16 ) & 0xFF,
 *          ( addr >>  8 ) & 0xFF, ( addr       ) & 0xFF
 *          );
 *      sprintf( log_buf, "Sock.sinaddr:  %s", buf );
 *      log_string( log_buf );
 *      from = gethostbyaddr( (char *) &sock.sin_addr,
 *          sizeof(sock.sin_addr), AF_INET );
 *      dnew->host = str_dup( from ? from->h_name : buf );
 */
    }

    /*
     * Swiftest: I added the following to ban sites.  I don't
     * endorse banning of sites, but Copper has few descriptors now
     * and some people from certain sites keep abusing access by
     * using automated 'autodialers' and leaving connections hanging.
     *
     * Furey: added suffix check by request of Nickel of HiddenWorlds.
     */
    if ( check_ban(dnew->host,BAN_ALL))
    {
	write_to_descriptor( desc,
	    "Your site has been banned from this mud.\n\r", 0 );
	#if defined(WIN32)
	closesocket( desc );
	#else
	close( desc );
	#endif
	free_descriptor(dnew);
	return;
    }
    /*
     * Init descriptor data.
     */
    dnew->next                  = descriptor_list;
    descriptor_list             = dnew;

    /*
     * Send the greeting.
     */
    {
	extern char * help_greeting;
	if ( help_greeting[0] == '.' )
	    write_to_buffer( dnew, help_greeting+1, 0 );
	else
	    write_to_buffer( dnew, help_greeting  , 0 );
    }

    return;
}
#endif



void close_socket( DESCRIPTOR_DATA *dclose, bool toast )
{
    CHAR_DATA *ch;
    char buf[MSL];

    if ( dclose->character && toast )
    {
	toast_corpses( dclose->character->name );
    }

#if defined(unix)
    if ( dclose->ipid > -1 ) 
    {
	int status;

	kill( dclose->ipid, SIGKILL );
	waitpid( dclose->ipid, &status, WNOHANG );
    }
    if ( dclose->ifd > -1 )
	close( dclose->ifd );
#endif

    if ( dclose->outtop > 0 )
        process_output( dclose, FALSE );

    if ( dclose->snoop_by != NULL )
    {
	write_to_buffer( dclose->snoop_by,
	    "Your victim has left the game.\n\r", 0 );
    }

    {
	DESCRIPTOR_DATA *d;

	for ( d = descriptor_list; d != NULL; d = d->next )
	{
	    if ( d->snoop_by == dclose )
	        d->snoop_by = NULL;
	}
    }

    if ( ( ch = dclose->character ) != NULL )
    {
	sprintf( log_buf, "Closing link to %s.", ch->name );
	log_string( log_buf );
	/* cut down on wiznet spam when rebooting */
	if ( dclose->connected == CON_PLAYING && !merc_down )
	{
	    act( "$n has lost $s link.", ch, NULL, NULL, TO_ROOM );

	    wiznet( "Net death has claimed $N.",
	        ch, NULL, WIZ_LINKS, 0, get_trust(ch) );

	    ch->desc = NULL;
	}
	else
	{
	    if ( !IS_NULLSTR(dclose->character->name) )
	    {
		sprintf( buf, "Net death has claimed %s.", dclose->character->name );
		wiznet( buf, NULL, NULL, WIZ_LINKS, 0, get_trust(dclose->character) );
	    }
	    else 
	        wiznet( "Net death has claimed an unnamed player.",
	            NULL, NULL, WIZ_LINKS, 0, 0 );

	    free_char(dclose->original ? dclose->original : 
	        dclose->character );
	}
    }

    if ( d_next == dclose )
	d_next = d_next->next;   

    if ( dclose == descriptor_list )
    {
	descriptor_list = descriptor_list->next;
    }
    else
    {
	DESCRIPTOR_DATA *d;

	for ( d = descriptor_list; d && d->next != dclose; d = d->next )
	    ;
	if ( d != NULL )
	    d->next = dclose->next;
	else
	    bug( "Close_socket: dclose not found.", 0 );
    }

    #if defined(WIN32)
    closesocket( dclose->descriptor );
    #else
    close( dclose->descriptor );
    #endif
    #if defined(unix)
    free_string( dclose->ident );
    #endif
    free_descriptor(dclose);
    #if defined(MSDOS)
    exit(1);
    #endif
    return;
}


bool read_from_descriptor( DESCRIPTOR_DATA *d )
{
    int iStart;

    /* Hold horses if pending command already. */
    if ( d->incomm[0] != '\0' )
	return TRUE;

    /* Check for overflow. */
    iStart = strlen(d->inbuf);
    if ( iStart >= sizeof(d->inbuf) - 10 )
    {
	sprintf( log_buf, "%s input overflow!", d->host );
	log_string( log_buf );
	write_to_descriptor( d->descriptor,
	    "\n\r*** PUT A LID ON IT!!! ***\n\r", 0 );
	return FALSE;
    }

    /* Snarf input. */
/*
#if defined(macintosh)
    for ( ; ; )
    {
	int c;
	c = getc( stdin );
	if ( c == '\0' || c == EOF )
	    break;
	putc( c, stdout );
	if ( c == '\r' )
	    putc( '\n', stdout );
	d->inbuf[iStart++] = c;
	if ( iStart > sizeof(d->inbuf) - 10 )
	    break;
    }
#endif
*/
//#if defined(MSDOS) || defined(unix)
    for ( ; ; )
    {
	int nRead;

#if defined(WIN32)
    nRead = recv( d->descriptor, d->inbuf + iStart,
        sizeof(d->inbuf) - 10 - iStart, 0 );
#else
	nRead = read( d->descriptor, d->inbuf + iStart,
	    sizeof(d->inbuf) - 10 - iStart );
#endif
	if ( nRead > 0 )
	{
	    iStart += nRead;
	    if ( d->inbuf[iStart-1] == '\n' || d->inbuf[iStart-1] == '\r' )
		break;
	}
	else if ( nRead == 0 )
	{
	    log_string( "EOF encountered on read." );
	    return FALSE;
	}
#if defined(WIN32)
    else if ( WSAGetLastError() == WSAEWOULDBLOCK )
#else
	else if ( errno == EWOULDBLOCK )
#endif
	    break;
	else
	{
	    perror( "Read_from_descriptor" );
	    return FALSE;
	}
    }
//#endif

    d->inbuf[iStart] = '\0';
    return TRUE;
}



/*
 * Transfer one line from input buffer to input line.
 */
void read_from_buffer( DESCRIPTOR_DATA *d )
{
    int i, j, k;

    /*
     * Hold horses if pending command already.
     */
    if ( d->incomm[0] != '\0' )
	return;

    /*
     * Look for at least one new line.
     */
    for ( i = 0; d->inbuf[i] != '\n' && d->inbuf[i] != '\r'; i++ )
    {
	if ( d->inbuf[i] == '\0' )
	    return;
    }

    /*
     * Canonical input processing.
     */
    for ( i = 0, k = 0; d->inbuf[i] != '\n' && d->inbuf[i] != '\r'; i++ )
    {
	if ( k >= MAX_INPUT_LENGTH - 2 )
	{
	    write_to_descriptor( d->descriptor, "Line too long.\n\r", 0 );

	    /* skip the rest of the line */
	    for ( ; d->inbuf[i] != '\0'; i++ )
	    {
		if ( d->inbuf[i] == '\n' || d->inbuf[i] == '\r' )
		    break;
	    }
	    d->inbuf[i]   = '\n';
	    d->inbuf[i+1] = '\0';
	    break;
	}

	if ( d->inbuf[i] == '\b' && k > 0 )
	    --k;
	else if ( isascii(d->inbuf[i]) && isprint(d->inbuf[i]) )
	    d->incomm[k++] = d->inbuf[i];
    }

    /*
     * Finish off the line.
     */
    if ( k == 0 )
	d->incomm[k++] = ' ';
    d->incomm[k] = '\0';

    /*
     * Deal with bozos with #repeat 1000 ...
     */

    if ( k > 1 || d->incomm[0] == '!' )
    {
	if ( d->incomm[0] != '!' && strcmp( d->incomm, d->inlast ) )
	{
	    d->repeat = 0;
	}
	else
	{
	    if (++d->repeat >= 25 && d->character
	    &&  d->connected == CON_PLAYING)
	    {
		sprintf( log_buf, "%s input spamming!", d->host );
		log_string( log_buf );
		wiznet("Spam spam spam $N spam spam spam spam spam!",
		       d->character,NULL,WIZ_SPAM,0,get_trust(d->character));
		if (d->incomm[0] == '!')
		    wiznet(d->inlast,d->character,NULL,WIZ_SPAM,0,
			get_trust(d->character));
		else
		    wiznet(d->incomm,d->character,NULL,WIZ_SPAM,0,
			get_trust(d->character));

		d->repeat = 0;
/*
		write_to_descriptor( d->descriptor,
		    "\n\r*** PUT A LID ON IT!!! ***\n\r", 0 );
		strcpy( d->incomm, "quit" );
*/
	    }
	}
    }


    /*
     * Do '!' substitution.
     */
    if ( d->incomm[0] == '!' )
	strcpy( d->incomm, d->inlast );
    else
	strcpy( d->inlast, d->incomm );

    /*
     * Shift the input buffer.
     */
    while ( d->inbuf[i] == '\n' || d->inbuf[i] == '\r' )
	i++;
    for ( j = 0; ( d->inbuf[j] = d->inbuf[i+j] ) != '\0'; j++ )
	;
    return;
}



/*
 * Low level output function.
 */
bool process_output( DESCRIPTOR_DATA *d, bool fPrompt )
{
    extern bool merc_down;

    /*
     * Bust a prompt.
     */
    if (!merc_down && d->showstr_point)
	write_to_buffer(d,"[Hit Return to continue]\n\r",0);
    else if (fPrompt && !merc_down && d->connected == CON_PLAYING)
    {
	CHAR_DATA *ch;
	CHAR_DATA *victim;

	ch = d->character;

	/* battle prompt */
	if ((victim = ch->fighting) != NULL && can_see(ch,victim))
	{
	    int percent;
	    char wound[100];
	    char buf[MAX_STRING_LENGTH];
 
	    if (victim->max_hit > 0)
		percent = victim->hit * 100 / victim->max_hit;
	    else
		percent = -1;
 
	    if (percent >= 100)
		sprintf(wound,"is in excellent condition.");
	    else if (percent >= 90)
		sprintf(wound,"has a few scratches.");
	    else if (percent >= 75)
		sprintf(wound,"has some small wounds and bruises.");
	    else if (percent >= 50)
		sprintf(wound,"has quite a few wounds.");
	    else if (percent >= 30)
		sprintf(wound,"has some big nasty wounds and scratches.");
	    else if (percent >= 15)
		sprintf(wound,"looks pretty hurt.");
	    else if (percent >= 0)
		sprintf(wound,"is in awful condition.");
	    else
		sprintf(wound,"is bleeding to death.");
 
	    sprintf( buf, "%s%s %s%s\n\r", 
		     P_COLOR(ch, C_COMBAT_SELF),
                     PERS(victim, ch),
                     wound,
                     P_COLOR_OFF(ch) );
	    buf[0] = UPPER(buf[0]);
	    write_to_buffer( d, buf, 0);
	}


	ch = d->original ? d->original : d->character;
	if (!IS_SET(ch->comm, COMM_COMPACT) )
	    write_to_buffer( d, "\n\r", 2 );


	if ( IS_SET(ch->comm, COMM_PROMPT) )
	    bust_a_prompt( d->character );

	if (IS_SET(ch->comm,COMM_TELNET_GA))
	    write_to_buffer(d,go_ahead_str,0);
    }

    /*
     * Short-circuit if nothing to write.
     */
    if ( d->outtop == 0 )
	return TRUE;

    /*
     * Snoop-o-rama.
     */
    if ( d->snoop_by != NULL )
    {
	if (d->character != NULL)
	    write_to_buffer( d->snoop_by, d->character->name,0);
	write_to_buffer( d->snoop_by, "> ", 2 );
	write_to_buffer( d->snoop_by, d->outbuf, d->outtop );
    }

    /*
     * OS-dependent output.
     */
    if ( !write_to_descriptor( d->descriptor, d->outbuf, d->outtop ) )
    {
	d->outtop = 0;
	return FALSE;
    }
    else
    {
	d->outtop = 0;
	return TRUE;
    }
}

/*
 * Bust a prompt (player settable prompt)
 * coded by Morgenes for Aldara Mud
 */
void bust_a_prompt( CHAR_DATA *ch )
{
    char buf[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    char prePrompt[MAX_STRING_LENGTH];
    const char *str;
    const char *i;
    char *point;
    char doors[MAX_INPUT_LENGTH];
    EXIT_DATA *pexit;
    bool found;
    const char *dir_name[] = {"N","E","S","W","U","D","Ne","Se","Sw","Nw","Cw","Ccw"};
    int door;
    
    prePrompt[0] = '\0';
    
    if ( !IS_NPC(ch)
    &&   !IS_AFFECTED2(ch, AFF2_WATER_BREATHING)
    &&   UNDERWATER(ch)
    &&   ch->level <= LEVEL_HERO )
	strcat( prePrompt, "[DROWNING] " );

    if ( !IS_NPC(ch)
    &&   IS_SET(ch->act, PLR_SINGING) )
	strcat( prePrompt, "[SINGING] " );
 
    point = buf;
    str = ch->prompt;
    if (str == NULL || str[0] == '\0')
    {
	sprintf( buf, "%s%s<%dhp %dm %dmv> %s%s",
	    P_COLOR(ch, C_PROMPT),
	    prePrompt,
	    ch->hit, ch->mana, ch->move, ch->prefix,
	    P_COLOR_LITERAL(ch, COLOR_OFF) );
	send_to_char(buf,ch);
	return;
    }

   if (IS_SET(ch->comm,COMM_AFK))
   {
	sprintf( buf, "%s%s<AFK>%s ", 
	    P_COLOR(ch, C_PROMPT), prePrompt, P_COLOR_LITERAL(ch,COLOR_OFF) );
	send_to_char(buf, ch);
	return;
   }

   while( *str != '\0' )
   {
      if( *str != '%' )
      {
	 *point++ = *str++;
	 continue;
      }
      ++str;
      switch( *str )
      {
	 default :
	    i = " "; break;
	case 'e':
	    found = FALSE;
	    doors[0] = '\0';
	    for (door = 0; door < 12; door++)
	    {
		/* Murgen - changed to limit on blindness. */
		if ((pexit = ch->in_room->exit[door]) != NULL
		&&  pexit ->u1.to_room != NULL
		&&  (can_see_room(ch,pexit->u1.to_room)
		||   IS_AFFECTED(ch,AFF_INFRARED))
		&&  !IS_AFFECTED(ch,AFF_BLIND)
		&&  !IS_SET(pexit->exit_info,EX_CLOSED))
		{
		    found = TRUE;
		    strcat(doors,dir_name[door]);
		}
	    }
	    if (!found)
		strcat(buf,"none");
	    sprintf(buf2,"%s",doors);
	    i = buf2; break;
	 case 'c' :
	    sprintf(buf2,"%s","\n\r");
	    i = buf2; break;
	 case 'h' :
	    sprintf( buf2, "%d", ch->hit );
	    i = buf2; break;
	 case 'H' :
	    sprintf( buf2, "%d", ch->max_hit );
	    i = buf2; break;
	 case 'm' :
	    sprintf( buf2, "%d", ch->mana );
	    i = buf2; break;
	 case 'M' :
	    sprintf( buf2, "%d", ch->max_mana );
	    i = buf2; break;
	 case 'v' :
	    sprintf( buf2, "%d", ch->move );
	    i = buf2; break;
	 case 'V' :
	    sprintf( buf2, "%d", ch->max_move );
	    i = buf2; break;
	 case 'x' :
	    sprintf( buf2, "%d", ch->exp );
	    i = buf2; break;
	 case 'X' :
	    sprintf(buf2, "%d", 
		(IS_NPC(ch) || ch->level == LEVEL_HERO)
		  ? 0 :
	    	  (ch->level + 1) * 
	    	    exp_per_level(ch,ch->pcdata->points) - ch->exp);
	    i = buf2; break;
	 case 'g' :
	    sprintf( buf2, "%ld", ch->gold);
	    i = buf2; break;
	 case 's' :
	    sprintf( buf2, "%ld", ch->silver);
	    i = buf2; break;
	 case 'a' :
	    if( ch->level > 9 )
	       sprintf( buf2, "%d", ch->alignment );
	    else
	       sprintf( buf2, "%s", IS_GOOD(ch) ? "good" : IS_EVIL(ch) ?
		"evil" : "neutral" );
	    i = buf2; break;
	 case 'r' :
	    if( ch->in_room != NULL )
	       sprintf( buf2, "%s", 
		(has_holylight(ch) ||
		 (!IS_AFFECTED(ch,AFF_BLIND) && !room_is_dark( ch, ch->in_room )))
		? ch->in_room->name : "darkness");
	    else
	       sprintf( buf2, " " );
	    i = buf2; break;
	 case 'R' :
	    if( IS_IMMORTAL( ch ) && ch->in_room != NULL )
	       sprintf( buf2, "%d", ch->in_room->vnum );
	    else
	       sprintf( buf2, " " );
	    i = buf2; break;
	 case 'z' :
	    if( IS_IMMORTAL( ch ) && ch->in_room != NULL )
	       sprintf( buf2, "%s", ch->in_room->area->name );
	    else
	       sprintf( buf2, " " );
	    i = buf2; break;
	 case '%' :
	    sprintf( buf2, "%%" );
	    i = buf2; break;
      }
      ++str;
      while( (*point = *i) != '\0' )
	 ++point, ++i;
   }
   write_to_buffer( ch->desc, P_COLOR(ch, C_PROMPT), 0 );
   write_to_buffer( ch->desc, prePrompt, 0 );
   write_to_buffer( ch->desc, buf, point - buf );
   write_to_buffer( ch->desc, P_COLOR_LITERAL(ch, COLOR_OFF), 0 );

   if (ch->prefix[0] != '\0')
	write_to_buffer(ch->desc,ch->prefix,0);
   return;
}



/*
 * Append onto an output buffer.
 */
void write_to_buffer( DESCRIPTOR_DATA *d, const char *txt, int length )
{
    /*
     * Find length in case caller didn't.
     */
    if ( length <= 0 )
	length = strlen(txt);

    /*
     * Initial \n\r if needed.
     */
    if ( d->outtop == 0 && !d->fcommand )
    {
	d->outbuf[0]    = '\n';
	d->outbuf[1]    = '\r';
	d->outtop       = 2;
    }

    /*
     * Expand the buffer as needed.
     */
    while ( d->outtop + length >= d->outsize )
    {
	char *outbuf;

	if (d->outsize >= 32000)
	{
	    bug("Buffer overflow. Closing.\n\r",0);
	    close_socket(d, TRUE);
	    return;
	}
	outbuf      = alloc_mem( 2 * d->outsize );
	strncpy( outbuf, d->outbuf, d->outtop );
	free_mem( d->outbuf, d->outsize );
	d->outbuf   = outbuf;
	d->outsize *= 2;
    }

    /*
     * Copy.
     */
    strncpy( d->outbuf + d->outtop, txt, length );
    d->outtop += length;
    return;
}



/*
 * Lowest level output function.
 * Write a block of text to the file descriptor.
 * If this gives errors on very long blocks (like 'ofind all'),
 *   try lowering the max block size.
 */
bool write_to_descriptor( int desc, char *txt, int length )
{
    int iStart;
    int nWrite;
    int nBlock;

#if defined(macintosh) || defined(MSDOS)
    if ( desc == 0 )
	desc = 1;
#endif

    if ( length <= 0 )
	length = strlen(txt);

    for ( iStart = 0; iStart < length; iStart += nWrite )
    {
	    nBlock = UMIN( length - iStart, 4096 );
	    #if defined(WIN32)
	    if ( ( nWrite = send ( desc, txt + iStart, nBlock, 0 ) ) < 0 )
	    #else
	    if ( ( nWrite = write( desc, txt + iStart, nBlock ) ) < 0 )
	    #endif
	    { 
	        perror( "Write_to_descriptor" ); 
	        return FALSE; 
	    }
    } 

    return TRUE;
}


/*
 * Load the invalid name parts global variable from the file.
 */
void load_invalid_name_parts( void )
{
	FILE *stream;
	
	if ( ( stream = fopen( INVALID_NAME_PARTS, "r" ) ) == NULL )
	{
	    perror( INVALID_NAME_PARTS );
	    invalid_name_parts = "";
	    return;
	}

	invalid_name_parts = fread_string_eol( stream );
	
	fclose(stream);
}


void stop_idling( CHAR_DATA *ch )
{
    if ( ch == NULL
    ||   ch->desc == NULL
    ||   ch->desc->connected != CON_PLAYING
    ||   ch->was_in_room == NULL 
    ||   ch->in_room != get_room_index(ROOM_VNUM_LIMBO))
	return;

    ch->timer = 0;
    char_from_room( ch, TRUE );
    char_to_room( ch, ch->was_in_room );
    ch->was_in_room     = NULL;
    act( "$n has returned from the void.", ch, NULL, NULL, TO_ROOM );
    return;
}



/*
 * Write to one char.
 */
void send_to_char( const char *txt, CHAR_DATA *ch )
{
    if ( txt != NULL && ch->desc != NULL )
	write_to_buffer( ch->desc, txt, strlen(txt) );
    return;
}

/*
 * Send a page to one char.
 */
void page_to_char( const char *txt, CHAR_DATA *ch )
{
    if ( txt == NULL || ch->desc == NULL)
	return;

    if (ch->lines == 0 )
    {
	send_to_char(txt,ch);
	return;
    }
	
#if defined(macintosh)
	send_to_char(txt,ch);
#else
    ch->desc->showstr_head = alloc_mem(strlen(txt) + 1);
    strcpy(ch->desc->showstr_head,txt);
    ch->desc->showstr_point = ch->desc->showstr_head;
    show_string(ch->desc,"");
#endif
}


/* string pager */
void show_string(struct descriptor_data *d, char *input)
{
    char buffer[4*MAX_STRING_LENGTH];
    char buf[MAX_INPUT_LENGTH];
    register char *scan, *chk;
    int lines = 0, toggle = 1;
    int show_lines;

    one_argument(input,buf);
    if (buf[0] != '\0')
    {
	if (d->showstr_head)
	{
	    free_mem(d->showstr_head,strlen(d->showstr_head));
	    d->showstr_head = 0;
	}
	d->showstr_point  = 0;
	return;
    }

    if (d->character)
	show_lines = d->character->lines;
    else
	show_lines = 0;

    for (scan = buffer; ; scan++, d->showstr_point++)
    {
	if (((*scan = *d->showstr_point) == '\n' || *scan == '\r')
	    && (toggle = -toggle) < 0)
	    lines++;

	else if (!*scan || (show_lines > 0 && lines >= show_lines))
	{
	    *scan = '\0';
	    write_to_buffer(d,buffer,strlen(buffer));
	    for (chk = d->showstr_point; isspace(*chk); chk++);
	    {
		if (!*chk)
		{
		    if (d->showstr_head)
		    {
			free_mem(d->showstr_head,strlen(d->showstr_head));
			d->showstr_head = 0;
		    }
		    d->showstr_point  = 0;
		}
	    }
	    return;
	}
    }
    return;
}
	

/* quick sex fixer */
void fix_sex(CHAR_DATA *ch)
{
    if (ch->sex < 0 || ch->sex > 2)
	ch->sex = IS_NPC(ch) ? 0 : ch->pcdata->true_sex;
}


char *act_string( CHAR_DATA *to, const char *format, CHAR_DATA *ch,
		  const void *arg1, const void *arg2, int *length )
{
    static char * const he_she  [] = { "it",  "he",  "she" };
    static char * const him_her [] = { "it",  "him", "her" };
    static char * const his_her [] = { "its", "his", "her" };
 
    static char buf[MAX_STRING_LENGTH];
    char fname[MAX_INPUT_LENGTH];
    CHAR_DATA *vch = (CHAR_DATA *) arg2;
    OBJ_DATA *obj1 = (OBJ_DATA  *) arg1;
    OBJ_DATA *obj2 = (OBJ_DATA  *) arg2;
    const char *str;
    const char *i;
    char *point;
    
    point   = buf;
    str     = format;
    while ( *str != '\0' )
    {
	if ( *str != '$' )
	{
	    *point++ = *str++;
	    continue;
	}
	++str;
 
	if ( arg2 == NULL && *str >= 'A' && *str <= 'Z' )
	{
	    bug( "Act: missing arg2 for code %d.", *str );
	    i = " <@@@> ";
	}
	else
	{
	    switch ( *str )
	    {
		default:  bug( "Act: bad code %d.", *str );
			  i = " <@@@> ";                                break;
		/* Thx alex for 't' idea */
		case 't': i = (char *) arg1;                            break;
		case 'T': i = (char *) arg2;                            break;
		case 'n': i = PERS( ch,  to  );                         break;
		case 'N': i = PERS( vch, to  );                         break;
		case 'e': i = he_she  [URANGE(0, ch  ->sex, 2)];        break;
		case 'E': i = he_she  [URANGE(0, vch ->sex, 2)];        break;
		case 'm': i = him_her [URANGE(0, ch  ->sex, 2)];        break;
		case 'M': i = him_her [URANGE(0, vch ->sex, 2)];        break;
		case 's': i = his_her [URANGE(0, ch  ->sex, 2)];        break;
		case 'S': i = his_her [URANGE(0, vch ->sex, 2)];        break;
 
		case 'p':
		    i = can_see_obj( to, obj1 )
			    ? obj1->short_descr
			    : "something";
		    break;
 
		case 'P':
		    i = can_see_obj( to, obj2 )
			    ? obj2->short_descr
			    : "something";
		    break;
 
		case 'd':
		    if ( arg2 == NULL || ((char *) arg2)[0] == '\0' )
		    {
			i = "door";
		    }
		    else
		    {
			one_argument( (char *) arg2, fname );
			i = fname;
		    }
		    break;
	    }
	}
 
	++str;
	while ( ( *point = *i ) != '\0' )
	    ++point, ++i;
    }
 
    *point++ = '\n';
    *point++ = '\r';
    *point   = '\0';
    buf[0]   = UPPER( buf[0] );

    *length  = point - buf;
    return buf;
}


int act (const char *format, CHAR_DATA *ch, const void *arg1, const void *arg2,
	  int type)
{
    /* to be compatible with older code */
    return act_new(format,ch,arg1,arg2,type,POS_RESTING);
}

int act_color( const char *format, CHAR_DATA *ch, const void *arg1, 
	      const void *arg2, int type, int min_pos, int color )
{
    CHAR_DATA *vch = (CHAR_DATA *) arg2;
    char buf2[MAX_STRING_LENGTH];
    int triggered = 0;
    CHAR_DATA *to;
    int txtlen;
    char *txt;
 
    /*
     * Discard null and zero-length messages.
     */
    if ( format == NULL || format[0] == '\0' )
	return 0;

    /* discard null rooms and chars */
    if (ch == NULL || ch->in_room == NULL)
	return 0;

    to = ch->in_room->people;
    if ( type == TO_VICT )
    {
	if ( vch == NULL )
	{
	    bug( "Act: null vch with TO_VICT.", 0 );
	    return 0;
	}

	if (vch->in_room == NULL)
	    return 0;

	to = vch->in_room->people;
    }

    if ( MOBtrigger && type != TO_CHAR && type != TO_VICT && to != NULL )
    {
	OBJ_DATA *to_obj;

	txt = act_string( to, format, ch, arg1, arg2, &txtlen);

	if ( ROOM_HAS_TRIGGER(to->in_room, RTRIG_ACT) )
	    rp_act_trigger( txt, to->in_room, ch, arg1, arg2, RTRIG_ACT );

	for ( to_obj = to->in_room->contents;
	      to_obj != NULL;
	      to_obj = to_obj->next_content )
	    if ( HAS_TRIGGER(to_obj, OTRIG_ACT) )
		op_act_trigger( txt, to_obj, ch, arg1, arg2, OTRIG_ACT );
    }

    for ( ; to != NULL; to = to->next_in_room )
    {
	if ( (to->desc == NULL
	&&    (!IS_NPC(to) || !HAS_TRIGGER(to, TRIG_ACT)))
	||   to->position < min_pos )
	    continue;
/* WW - removed this buggy version from mob program patch.
 *      if ( (!IS_NPC(to) && to->desc == NULL )
 *      ||   ( IS_NPC(to) && !HAS_TRIGGER(to, TRIG_ACT) )
 *      ||    to->position < min_pos )
 *          continue;
 */

	if ( (type == TO_CHAR) && to != ch )
	    continue;
	if ( type == TO_VICT && ( to != vch || to == ch ) )
	    continue;
	if ( type == TO_ROOM && to == ch )
	    continue;
	if ( type == TO_NOTVICT && (to == ch || to == vch) )
	    continue;

	txt = act_string( to, format, ch, arg1, arg2, &txtlen );
	sprintf( buf2, "%s%s%s",
	    P_COLOR(to, color), txt, P_COLOR_LITERAL(to, COLOR_OFF) );
	if ( to->desc != NULL )
	    write_to_buffer( to->desc, buf2 , 0 );
	else if ( MOBtrigger )
	{
	    if ( mp_act_trigger( txt, to, ch, arg1, arg2, TRIG_ACT) )
		if ( to == ch )
		    SET_BIT( triggered, TRIGGERED_CHAR );
		else if ( to == (CHAR_DATA *) arg2 )
		    SET_BIT( triggered, TRIGGERED_VICT );
		else
		    SET_BIT( triggered, TRIGGERED_ROOM );
	}
    }
    return triggered;
}

int act_new( const char *format, CHAR_DATA *ch, const void *arg1, 
	      const void *arg2, int type, int min_pos)
{
    CHAR_DATA *vch = (CHAR_DATA *) arg2;
    int triggered = 0;
    CHAR_DATA *to;
    int txtlen;
    char *txt;
 
    /*
     * Discard null and zero-length messages.
     */
    if ( format == NULL || format[0] == '\0' )
	return 0;

    /* discard null rooms and chars */
    if (ch == NULL || ch->in_room == NULL)
	return 0;

    to = ch->in_room->people;
    if ( type == TO_VICT )
    {
	if ( vch == NULL )
	{
	    bug( "Act: null vch with TO_VICT.", 0 );
	    return 0;
	}

	if (vch->in_room == NULL)
	    return 0;

	to = vch->in_room->people;
    }

    if ( MOBtrigger && type != TO_CHAR && type != TO_VICT && to != NULL )
    {
	OBJ_DATA *to_obj;

	txt = act_string( to, format, ch, arg1, arg2, &txtlen);

	if ( ROOM_HAS_TRIGGER(to->in_room, RTRIG_ACT) )
	    rp_act_trigger( txt, to->in_room, ch, arg1, arg2, RTRIG_ACT );

	for ( to_obj = to->in_room->contents;
	      to_obj != NULL;
	      to_obj = to_obj->next_content )
	    if ( HAS_TRIGGER(to_obj, OTRIG_ACT) )
		op_act_trigger( txt, to_obj, ch, arg1, arg2, OTRIG_ACT );
    }

    for ( ; to != NULL; to = to->next_in_room )
    {
	if ( (to->desc == NULL
	&&    (!IS_NPC(to) || !HAS_TRIGGER(to, TRIG_ACT)))
	||   to->position < min_pos )
	    continue;
/* WW - removed this buggy version from mob program patch.
 *      if ( (!IS_NPC(to) && to->desc == NULL )
 *      ||   ( IS_NPC(to) && !HAS_TRIGGER(to, TRIG_ACT) )
 *      ||    to->position < min_pos )
 *          continue;
 */
 
	if ( (type == TO_CHAR) && to != ch )
	    continue;
	if ( type == TO_VICT && ( to != vch || to == ch ) )
	    continue;
	if ( type == TO_ROOM && to == ch )
	    continue;
	if ( type == TO_NOTVICT && (to == ch || to == vch) )
	    continue;

	txt = act_string( to, format, ch, arg1, arg2, &txtlen );
	if ( to->desc != NULL )
	    write_to_buffer( to->desc, txt, txtlen );
	else if ( MOBtrigger )
	{
	    if ( mp_act_trigger( txt, to, ch, arg1, arg2, TRIG_ACT) )
		if ( to == ch )
		    SET_BIT( triggered, TRIGGERED_CHAR );
		else if ( to == (CHAR_DATA *) arg2 )
		    SET_BIT( triggered, TRIGGERED_VICT );
		else
		    SET_BIT( triggered, TRIGGERED_ROOM );
	}
    }
    return triggered;
}


void toast_corpses_by_type( char *name, int vnum )
{
    OBJ_DATA *obj, *obj_next;
    
    for ( obj = get_obj_index(vnum)->instances; 
	  obj != NULL; 
	  obj = obj_next )
    {
	obj_next = obj->next_instance;

	if ( obj->item_type == ITEM_CORPSE_PC
	&&   !str_cmp(obj->owner, name)
	&&   obj->in_room != NULL  )
	{
            if ( obj->in_room->people != NULL )
            {
	    act( "$p sinks into the ground.",
		obj->in_room->people, obj, NULL, TO_ROOM );
	    act( "$p sinks into the ground.",
		obj->in_room->people, obj, NULL, TO_CHAR );
            }
	    extract_obj( obj );
	}
    }
}


void toast_corpses( char *name )
{
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC_BASH );
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC_FIRE );
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC_COLD );
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC_LIGHTNING );
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC_ACID );
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC_POISON );
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC_NEGATIVE );
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC_HOLY );
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC_ENERGY );
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC_DISEASE );
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC_DROWNING );
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC_SLAY );
    toast_corpses_by_type( name, OBJ_VNUM_CORPSE_PC );

    /* Because of our function naming this is ugly but we
       also need to toast our owned equipment. The list of
       vnums of owned items is well known so we just go
       through them here. */
    toast_corpses_by_type( name, OBJ_VNUM_DRUID_STAFF );
    toast_corpses_by_type( name, OBJ_VNUM_SHADOWBLADE );
    toast_corpses_by_type( name, OBJ_VNUM_DEFENDER );
}


#if defined(unix)
void my_sig_handler( int signum)
{
    char buf[MAX_STRING_LENGTH];
    int status;
    pid_t pid;
  
    if ( (pid = wait(&status)) < 0 )
    {
       /*perror( "My_sig_handler : wait : " );*/
    }
    else
    {
       sprintf( buf, "reaped child %d", pid );
       log_string( buf );
    }
}
#endif


void do_abort()
{
#if defined(unix)
    pid_t pid = fork();

    if ( pid < 0 )
	bug( "Do_abort: fork failed.", 0 );
    else if ( pid == 0 )
	abort();
    else
    {
	waitpid( pid, NULL, 0 );
	system( CORE_EXAMINE_SCRIPT );
    }
#endif
}


/*
 * Macintosh support functions.
 */
///#if defined(macintosh)
///int gettimeofday( struct timeval *tp, void *tzp )
///{
///    tp->tv_sec  = time( NULL );
///    tp->tv_usec = 0;
///}
///#endif

/*
 * Win32 support functions.
 */
#if defined(WIN32)
void gettimeofday( struct timeval *tp, void *tzp )
{
    struct _timeb ft;
    
    tp->tv_sec  = time( NULL );
    
    _ftime( &ft );
    tp->tv_usec = ft.millitm;    
}
#endif

void logf (char * fmt, ...)
{
	char buf [2*MSL];
	va_list args;
	va_start (args, fmt);
	vsprintf (buf, fmt, args);
	va_end (args);
	
	log_string (buf);
}
