/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "merc.h"
#include "olc.h"
#include "magic.h"
#include "color.h"
#include "tables.h"
#include "recycle.h"


DECLARE_DO_FUN( do_follow           );

extern char *target_name;


void remove_all_equipment( CHAR_DATA *ch )
{
    OBJ_DATA *current;
    for ( current = ch->carrying; current != NULL; current = current->next_content )
    {
        if ( current->wear_loc != WEAR_NONE )
        {
            unequip_char( ch, current );
            printf( "Removing %s\n", current->short_descr );
        }
    }
}


int calc_mob_hits( int level )
{
    int hits;

    level = URANGE( 1, level, 60 );
    hits = dice( mobile_autos[level-1].hits[0] ,mobile_autos[level-1].hits[1] ) +
           mobile_autos[level-1].hits[2];
    return hits;
}


void polymorph_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( ch == NULL )
        return;

    if ( reason == AFF_PROC_FINISHED )
    {
        int max_hits;
        if ( IS_NPC(ch) )
            max_hits = dice( ch->pIndexData->hit[DICE_NUMBER],
                             ch->pIndexData->hit[DICE_TYPE] ) +
                       ch->pIndexData->hit[DICE_BONUS];
        else
            max_hits = ch->pcdata->perm_hit;

        ch->hit = UMAX( 1, ch->hit * max_hits / calc_mob_hits( paf->level ) );
    }
}


void level_familier( CHAR_DATA *familier, int level )
{
    familier->level             = level;
    familier->hit               = calc_mob_hits( level );
    familier->max_hit           = familier->hit;
    familier->hitroll           = level;
    familier->damroll           = level;
}


void spell_banish_dazzle( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) )
    {
	if ( victim == ch )
	  send_to_char( "You're already protected from the harsh light of the sun!\n\r", ch );
	else
	  send_to_char( "They're already protected from the harsh light of the sun!\n\r", ch );
	return;
    }

    af.where		= TO_RESIST;
    af.type		= sn;
    af.level		= level;
    af.duration		= 2 * level;
    af.location		= 0;
    af.modifier         = APPLY_NONE;
    af.bitvector	= RES_LIGHT;
    af.procNo		= 0;
    af.flags		= 0;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );

    send_to_char( "You're protected from the harsh light of the sun.\n\r", victim );
    if ( ch != victim )
        send_to_char( "You protect them the harsh light of the sun.\n\r", ch );
}


void spell_sunfire( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    static const sh_int dam_each[] = 
    {
	  0,
	  0,   0,   0,   0,   0,	  0,   0,   0,   0,   0,
	  0,   0,   0,   0,  30,	 35,  40,  45,  50,  55,
	 60,  65,  70,  75,  80,	 82,  84,  86,  88,  90,
	 92,  94,  96,  98, 100,	102, 104, 106, 108, 110,
	112, 114, 116, 118, 120,	122, 124, 126, 128, 130
    };
    int dam;
    
    if ( UNDERWATER(ch) )
    {
	send_to_char( "Nothing seems to happen.\n\r", ch );
	return;
    }

    level	= UMIN( level, sizeof(dam_each)/sizeof(dam_each[0]) - 1 );
    level	= UMAX( 0, level );
    dam		= number_range( dam_each[level] / 2, dam_each[level] * 2 );
    if ( saves_spell(level, victim, DAM_LIGHT) )
	dam /= 2;
    fire_effect( victim, level, dam/2, TARGET_CHAR );
    damage_old( ch, victim, dam, sn, DAM_LIGHT ,TRUE, NULL );
    return;
}


void spell_sundazzle( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    CHAR_DATA *vch;
    int hits = 0;

    if ( room_is_dark(ch, ch->in_room) )
    {
        send_to_char( "Nothing seems to happen!\n\r", ch );
        return;
    }

    for ( vch = ch->in_room->people; vch != NULL && hits <= level / 10; vch = vch->next_in_room )
    {
        AFFECT_DATA af;
        
        if ( !can_see(ch, vch)
        ||   (vch->fighting != ch 
        &&    vch != victim) )
            continue;

        if ( saves_spell(level, vch, DAM_LIGHT) )
        {
            act( "A small sphere of blinding light shoots from your hand, hitting $N full in the face!", ch, NULL, vch, TO_CHAR );
            act( "A small sphere of blinding light shoots from $ns hand, hitting YOU full in the face!", ch, NULL, vch, TO_VICT );
            act( "A small sphere of blinding light shoots from $ns hand, hitting $N full in the face!", ch, NULL, vch, TO_NOTVICT );
            continue;
        }

        ++hits;

        act( "A small sphere of blinding light shoots from your hand, hitting $N full in the face!", ch, NULL, vch, TO_CHAR );
        act( "A small sphere of blinding light shoots from $ns hand, hitting YOU full in the face!", ch, NULL, vch, TO_VICT );
        act( "A small sphere of blinding light shoots from $ns hand, hitting $N full in the face!", ch, NULL, vch, TO_NOTVICT );

        if ( IS_AFFECTED(vch, AFF_BLIND) )
            continue;

        af.where        = TO_AFFECTS;
        af.type         = sn;
        af.level        = level;
        af.location     = APPLY_NONE;
        af.modifier     = 0;
        af.duration     = level / 10;
        af.bitvector	= AFF_BLIND;
        af.procNo       = 0;
        af.flags        = 0;
        af.name         = "";
        af.appearance   = "";
        affect_to_char( victim, &af );

        send_to_char( "You are blinded!\n\r", vch );
        act("$n appears to be blinded.",victim,NULL,NULL,TO_ROOM);
    }
}


/* =================================================================== */


void spell_traceless_travel( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) )
    {
	if ( victim == ch )
	  send_to_char( "Your tracks are already masked.\n\r", ch );
	else
	  send_to_char( "They're tracks are already masked.\n\r", ch );
	return;
    }

    af.where		= TO_AFFECTS2;
    af.type		= sn;
    af.level		= level;
    af.duration		= 2 * level;
    af.location		= 0;
    af.modifier         = APPLY_NONE;
    af.bitvector	= AFF2_TRACELESS_TRAVEL;
    af.procNo		= 0;
    af.flags		= 0;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );

    send_to_char( "Your tracks are now masked from those who would follow.\n\r", victim );
    if ( ch != victim )
        send_to_char( "You protect their tracks from those who would follow.\n\r", ch );
}


void spell_sand_shadow( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) )
    {
	if ( victim == ch )
	  send_to_char( "You are already sneaking.\n\r", ch );
	else
	  send_to_char( "They are already sneaking.\n\r", ch );
	return;
    }

    af.where		= TO_AFFECTS;
    af.type		= sn;
    af.level		= level;
    af.duration		= level / 15;
    af.location		= 0;
    af.modifier         = APPLY_NONE;
    af.bitvector	= AFF_SNEAK;
    af.procNo		= 0;
    af.flags		= 0;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );

    send_to_char( "You are cloaked in shadow.\n\r", victim );
    if ( ch != victim )
        send_to_char( "You cloak them in shadow.\n\r", ch );
}


void spell_wind_shadow( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) )
    {
	if ( victim == ch )
	  send_to_char( "You are already one with the wind.\n\r", ch );
	else
	  send_to_char( "They are already one with the wind.\n\r", ch );
	return;
    }

    af.where		= TO_AFFECTS;
    af.type		= sn;
    af.level		= level;
    af.duration		= 1;
    af.location		= 0;
    af.modifier         = APPLY_NONE;
    af.bitvector	= AFF_SNEAK + AFF_PASS_DOOR + AFF_FLYING;
    af.procNo		= 0;
    af.flags		= 0;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );

    send_to_char( "You become one with the wind!\n\r", victim );
    if ( ch != victim )
        send_to_char( "OK.\n\r", ch );
}


void spell_fire_track( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *tracker;
    AFFECT_DATA af;
    int door;

    if ( ch->in_room->sector_type == SECT_WATER_SWIM
    ||   ch->in_room->sector_type == SECT_WATER_NOSWIM
    ||   ch->in_room->sector_type == SECT_AIR )
    {
        send_to_char( "Nothing seems to happen. Perhaps there is not enough earth here.\n\r",
            ch );
        return;
    }

    tracker         = create_mobile( get_mob_index(MOB_VNUM_FIRE_TRACKER) );
    act( "A rift in space opens, a deep red light glowing through from beyond. A small flame floats through.",
         ch, NULL, NULL, TO_ROOM );
    act( "A rift in space opens, a deep red light glowing through from beyond. A small flame floats through.",
         ch, NULL, NULL, TO_CHAR );
    char_to_room( tracker, ch->in_room );

    do_follow( ch, "flame" );

    door = find_first_step(
        tracker,
        target_name,
        TRACK_NAME,
        UMAX( 20, level ),
        TRUE,
        TRUE,
        FALSE );

    if ( door == BFS_ERROR || door == BFS_NO_PATH || door == BFS_FOUND )
        return;

    af.where            = TO_AFFECTS2;
    af.type             = sn;
    af.level            = UMAX( 20, level );
    af.duration         = -1;
    af.location         = 0;
    af.modifier         = 0;
    af.bitvector        = AFF2_FIRE_TRACK;
    af.procNo           = affect_proc_lookup("fire track");
    af.flags            = AFF_CONT_UPDATE;
    af.name             = str_dup( target_name );
    af.appearance       = "";
    affect_to_char( tracker, &af );
}


void fire_track_affect_proc( CHAR_DATA *ch, OBJ_DATA *obj, AFFECT_DATA *paf, int reason )
{
    if ( ch == NULL )
        return;

    if ( reason == AFF_PROC_PERIODIC )
        damage_old( ch, ch, ch->max_hit / 4, paf->type,
                    DAM_NONE, FALSE, NULL );
}


/* =================================================================== */


void spell_contingency( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    char arg1[MAX_INPUT_LENGTH], arg2[MAX_INPUT_LENGTH];
    int modifier = 0;
    AFFECT_DATA af;
    bool stolen;
    int mana;
    int csn;

    if ( IS_AFFECTED2(ch, AFF2_CONTINGENCY) )
    {
        send_to_char( "You're already affected by contingency.\n\r", ch );
        return;
    }

    target_name = one_argument( target_name, arg1 );
                  one_argument( target_name, arg2 );
    if ( IS_NULLSTR(arg1) || IS_NULLSTR(arg2) )
    {
	send_to_char( "Syntax: cast contingency <attacked|robbed|drowning> <spell name>\n\r", ch );
	return;
    }
    
    if ( str_prefix(arg1, "attacked") == 0 )
	modifier = CONTIN_ATTACKED;
    else if ( str_prefix(arg1, "robbed") == 0 )
	modifier = CONTIN_ROBBED;
    else if ( str_prefix(arg1, "drowning") == 0 )
	modifier = CONTIN_DROWNING;
    else
    {
	send_to_char( "The triggering event must be either attacked, robbed, killed or drowning.\n\r", ch );
	return;
    }
    
    if ( !can_cast(ch, arg2, &csn, &stolen) )
    {
	send_to_char( "You don't know the secondary spell.\n\r", ch );
	return;
    }
    
    mana = get_mana_cost( ch, csn );
    if ( mana > ch->mana )
    {
	send_to_char( "You don't have enough mana to cast the secondary spell.\n\r", ch );
	return;
    }
    
    if ( number_percent() > get_skill(ch, csn) && !stolen )
    {
	send_to_char( "You lost your concentration casting the secondary spell.\n\r", ch );
	ch->mana -= mana / 2;
	return;
    }
    
    ch->mana -= mana;
    
    af.where            = TO_AFFECTS2;
    af.type             = sn;
    af.level            = level;
    af.duration         = -1;
    af.location         = 0;
    af.modifier         = modifier;
    af.bitvector        = AFF2_CONTINGENCY;
    af.procNo           = 0;
    af.flags            = AFF_INVIS;
    af.name             = skill_table[csn].name;
    af.appearance       = "";
    affect_to_char( ch, &af );

    send_to_char( "Contingency set.\n\r", ch );
}


void check_contingency( CHAR_DATA *ch, int type )
{
    if ( IS_AFFECTED2(ch, AFF2_CONTINGENCY) )
    {
        AFFECT_DATA *paf = affect_find( ch->affected, gsn_contingency );
        int sn;

        if ( paf == NULL )
        {
            bug( "check_contingency: unable to find contingency spell on %s.", (long) ch->name );
            REMOVE_BIT( ch->affected_by2, AFF2_CONTINGENCY );
            return;
        }

        if ( paf->modifier != type )
            return;
    
        sn = skill_lookup( paf->name );
        if ( sn == -1 )
        {
            bug( "check_contingency: unable to lookup stored spell on %s.", (long) ch->name );
            affect_strip( ch, gsn_contingency );
            return;
        }

        if ( skill_table[sn].spell_fun == spell_null )
        {
            bug( "check_contingency: contingency spell has null spell_fun. Spell name: %s.", (long) paf->name );
            affect_strip( ch, gsn_contingency );
            return;
        }

        send_to_char( "Your contingency spell activates.\n\r", ch );
        (*skill_table[sn].spell_fun) ( sn, paf->level, ch, ch, TAR_CHAR_SELF );
        affect_strip( ch, gsn_contingency );
    }
}


void spell_extend( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    char arg[MAX_INPUT_LENGTH];
    bool found = FALSE;
    AFFECT_DATA *paf;
    
    one_argument( target_name, arg );
    if ( IS_NULLSTR(arg) )
    {
        send_to_char( "Extend what spell?\n\r", ch );
        return;
    }
    
    for ( paf = ch->affected; paf; paf = paf->next )
    {
        if ( (paf->name[0] != '\0'
        &&    !str_prefix(arg, paf->name))
        ||   !str_prefix(arg, skill_table[paf->type].name) )
        {
            if ( paf->duration == -1
            ||   IS_SET(paf->flags, AFF_EXTENDED) )
            {
                if ( found )
                    continue;
                send_to_char( "That spell can't be extended.\n\r", ch );
                return;
            }
            
            found = TRUE;
            paf->flags    |= AFF_EXTENDED;
            paf->duration += level / 5; 
        }
    }
    
    if ( found )
        send_to_char( "You renew the power of the spell.\n\r", ch );
    else
        send_to_char( "You don't appear to be affected by that spell.\n\r", ch );
}


void spell_extend_true( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    AFFECT_DATA *paf;
    
    for ( paf = ch->affected; paf; paf = paf->next )
    {
        if ( paf->duration == -1
        || IS_SET(paf->flags, AFF_EXTENDED) )
            continue;
            
        paf->flags    |= AFF_EXTENDED;
        paf->duration += level / 5; 
    }
    
    send_to_char( "You renew the power of your spells.\n\r", ch );
}


void spell_spell_haste( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    
    if ( IS_AFFECTED2(victim, AFF2_SPELL_HASTE) )
    {
        if ( victim == ch )
            send_to_char( "You are already affected by spell haste.\n\r", ch );
        else
            send_to_char( "They are already affected by spell haste.\n\r", ch );
        return;
    }
    
    af.where            = TO_AFFECTS2;
    af.type             = sn;
    af.level            = level;
    af.duration         = level / 5;
    af.location         = 0;
    af.modifier         = 0;
    af.bitvector        = AFF2_SPELL_HASTE;
    af.procNo           = 0;
    af.flags            = 0;
    af.name             = "";
    af.appearance       = "";
    affect_to_char( victim, &af );

    send_to_char( "Your skill at magic improves.\n\r", victim ); 
    if ( ch != victim )
        send_to_char( "Their skill at magic improves.\n\r", victim ); 
}


/* =================================================================== */


void spell_release_mana( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *focus;
    for ( focus = ch->carrying; focus != NULL; focus = focus->next_content )
    {
        if ( focus->item_type != ITEM_FOCUS
        &&   focus->wear_loc == WEAR_NONE )
            continue;
        if ( focus->value[1] )
            break;
    }
    
    if ( focus == NULL )
    {
        send_to_char( "You need to be wearing a focus with stored mana for that to work!\n\r", ch );
        return;
    }
    
    if ( focus->value[2] != ch->id )
    {
        send_to_char( "You can only draw your own stored mana from a focus.\n\r", ch );
        return;
    }
    
    ch->mana += focus->value[1];
    focus->value[0] = UMAX( 0, focus->value[0] - 1 );
    focus->value[1] = 0;
    focus->value[2] = 0;

    act( "You drain the mana from $p, adding it to your own.", ch, focus, NULL, TO_CHAR );
    act( "$n drains the mana from $p, adding it to $s own.", ch, focus, NULL, TO_ROOM );
}


void spell_store_mana( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *focus = (OBJ_DATA *) vo;
    OBJ_DATA *obj;
    if ( focus->item_type != ITEM_FOCUS )
    {
        send_to_char( "You can only store mana in a focus.\n\r", ch );
        return;
    }
    
    if ( focus->value[1] )
    {
        send_to_char( "That focus already has mana stored in it.\n\r", ch );
        return;
    }
    
    for ( obj = object_list; obj != NULL; obj = obj->next )
    {
        if ( obj->item_type != ITEM_FOCUS
        ||   obj->value[1] == 0 )
            continue;
        if ( obj->value[2] == ch->id )
        {
            send_to_char( "You already have mana stored in a focus.\n\r", ch );
            return;
        }
    }
    
    if ( ch->mana == 0 )
    {
        send_to_char( "You don't have any mana left to store.\n\r", ch );
        return;
    }
    
    focus->value[1] = ch->mana;
    focus->value[2] = ch->id;
    ch->mana        = 0;
    
    act( "$p glows brightly as $n channels power into it.", ch, focus, NULL, TO_ROOM );
    act( "$p glows brightly as you channel power into it.", ch, focus, NULL, TO_ROOM );
}


void spell_overload_protection( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *focus = (OBJ_DATA *) vo;
    if ( focus->item_type != ITEM_FOCUS )
    {
        send_to_char( "You can only protect a focus from overload.\n\r", ch );
        return;
    }
    
    if ( IS_SET(focus->value[3], FOCUS_OVERLOAD_PROTECTED) )
    {
        send_to_char( "Your focus is already protected.\n\r", ch );
        return;
    }
    
    SET_BIT( focus->value[3], FOCUS_OVERLOAD_PROTECTED );
    
    act( "$p glows with a silvery aura.", ch, focus, NULL, TO_ALL );
} 


void spell_harden_focus( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *focus = (OBJ_DATA *) vo;
    if ( focus->item_type != ITEM_FOCUS )
    {
        send_to_char( "You can only harden a focus!\n\r", ch );
        return;
    }
    
    if ( IS_SET(focus->value[3], FOCUS_HARDENED) )
    {
        send_to_char( "Your focus is already protected.\n\r", ch );
        return;
    }
    
    SET_BIT( focus->value[3], FOCUS_HARDENED );
    
    act( "$p glows with a golden aura.", ch, focus, NULL, TO_ALL );
} 


/* =================================================================== */


void spell_extinguish( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    OBJ_DATA *obj;
    
    level += 10; /* Easier to blank out light sources temporarily than blind someone. */
    if ( saves_spell(level, victim, DAM_NEGATIVE) )
    {
        send_to_char( "Nothing seemed to happen.\n\r", ch );
        return;
    }
    
    for ( obj = victim->carrying; obj; obj = obj->next_content )
    {
        AFFECT_DATA af;
        
        if ( obj->item_type != ITEM_LIGHT
        ||   IS_OBJ_STAT(obj, ITEM_EXTINGUISHED) )
            continue;

        af.where                = TO_OBJECT;
        af.type                 = sn;
        af.level                = level;
        af.duration             = level;
        af.location             = 0;
        af.modifier             = 0;
        af.bitvector            = ITEM_EXTINGUISHED;
        af.procNo               = 0;
        af.flags                = 0;
        af.name                 = "";
        af.appearance           = "";
        affect_to_obj( obj, &af );
        
        act( "The light from $p is extinguished!", ch, obj, NULL, TO_ALL );
    }
}


void spell_backlash( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    
    if ( IS_AFFECTED2(ch, AFF2_BACKLASH) )
    {
        send_to_char( "They are already affected by backlash.\n\r", ch );
        return;
    }
    
    if ( saves_spell(level, victim, DAM_NEGATIVE) )
    {
        send_to_char( "Nothing seemed to happen.\n\r", ch );
        return;
    }
    
    af.where                    = TO_AFFECTS2;
    af.type                     = sn;
    af.level                    = level;
    af.duration                 = -1;
    af.location                 = 0;
    af.modifier                 = 0;
    af.bitvector                = AFF2_BACKLASH;
    af.procNo                   = 0;
    af.flags                    = AFF_INVIS;
    af.name                     = "";
    af.appearance               = "";
    affect_to_char( victim, &af );
    
    send_to_char( "They are cursed with your backlash!\n\r", ch );
}


void spell_weapon_curse( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    OBJ_DATA *blade = (OBJ_DATA *) vo;
    AFFECT_DATA af;
    
    if ( blade->item_type != ITEM_WEAPON )
    {
        send_to_char( "You can only cast dark blade on a weapon.\n\r", ch );
        return;
    }
    
    if ( IS_WEAPON_STAT(blade, WEAPON_CURSE) )
    {
        send_to_char( "Nothing seems to happen.\n\r", ch );
        return;
    }
    
    af.where                    = TO_WEAPON;
    af.type                     = sn;
    af.level                    = level / 2;
    af.duration                 = level / 10;
    af.location                 = 0;
    af.modifier                 = 0;
    af.bitvector                = WEAPON_CURSE;
    af.procNo                   = 0;
    af.flags                    = 0;
    af.name                     = "";
    af.appearance               = "";
    affect_to_obj( blade, &af );
    
    act( "$p is surrounded by an aura of inky blackness.", ch, blade, NULL, TO_ALL );
}


void spell_spider_form( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    bool replaceHits;
    AFFECT_DATA af;
    
    if ( victim->polymorphed )
    {
        if ( victim == ch )
            send_to_char( "You can't change form again.\n\r", ch );
        else
            send_to_char( "They can't change form again.\n\r", ch );
        return;
    }
    
    if ( ch == victim )
        act( "With a hideously tortured scream $n transforms $mself into a huge black spider!",
             ch, NULL, NULL, TO_ROOM );
    else
        act( "With a hideously tortured scream $n is transformed into a huge black spider!",
             victim, NULL, NULL, TO_ROOM );
    send_to_char( "An agonizing pain wracks your body as it changes shape!\n\r", ch );

    replaceHits                 = ch->hit * calc_mob_hits( level ) / ch->max_hit > ch->hit;
    if ( replaceHits )
        ch->hit                 = ch->hit * calc_mob_hits( level ) / ch->max_hit;

    af.where                    = TO_POLYMORPH;
    af.type                     = sn;
    af.level                    = level;
    af.duration                 = level / 5;
    af.location                 = APPLY_NONE;
    af.modifier                 = MOB_VNUM_SPIDER;
    af.bitvector                = 0;
    af.procNo                   = affect_proc_lookup("polymorph");
    af.flags                    = AFF_PROC_FINISHED;
    af.name                     = "";
    af.appearance               = "";
    affect_to_char( victim, &af );
    
    af.procNo                   = 0;
    af.flags                    = 0;
    
    af.where                    = TO_AFFECTS2;
    if ( replaceHits )
    {
    af.location                 = APPLY_HIT;
    af.modifier                 = calc_mob_hits( level ) - ch->max_hit;
    }
    af.bitvector                = AFF2_NO_EQUIPMENT + AFF2_CANT_CAST + AFF2_HALF_XP;
    af.flags                    = AFF_INVIS;
    affect_to_char( victim, &af );
    
    remove_all_equipment( ch );
}


void spell_poison_resistance( int sn, int level, CHAR_DATA *ch, void *vo,int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;

    if ( is_affected(victim, sn) )
    {
	if ( victim == ch )
	  send_to_char( "You're already resistant to poison!\n\r", ch );
	else
	  send_to_char( "They're already resistant to poison!\n\r", ch );
	return;
    }

    af.where		= TO_RESIST;
    af.type		= sn;
    af.level		= level;
    af.duration		= 2 * level;
    af.location		= 0;
    af.modifier         = APPLY_NONE;
    af.bitvector	= RES_POISON;
    af.procNo		= 0;
    af.flags		= 0;
    af.name		= "";
    af.appearance	= "";
    affect_to_char( victim, &af );

    send_to_char( "You're protected from poison.\n\r", victim );
    if ( ch != victim )
        send_to_char( "You protect them from poison.\n\r", ch );
}


void spell_spider_plague( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    AFFECT_DATA af;
    int try;
    
    if ( saves_spell(level, victim, DAM_NONE) )
    {
        act( "$n opens $s mouth, vomiting out a torrent of spiders which narrowly miss you!",
             ch, NULL, victim, TO_VICT );
        act( "$n opens $s mouth, vomiting out a torrent of spiders which narrowly miss $N!",
             ch, NULL, victim, TO_ROOM );
        act( "You vomit out a plague of spiders, narrowly missing $N!",
             ch, NULL, victim, TO_CHAR );
        return;
    }
    
    act( "$n opens $s mouth, vomiting out a torrent of spiders all over YOU!",
         ch, NULL, victim, TO_VICT );
    act( "$n opens $s mouth, vomiting out a torrent of spiders all over $N!",
         ch, NULL, victim, TO_ROOM );
    act( "You vomit out a plague of spiders all over $N!",
         ch, NULL, victim, TO_CHAR );

    af.where            = TO_AFFECTS;
    af.type             = sn;
    af.level            = level;
    af.duration         = 0;
    af.location         = APPLY_DEX;
    af.modifier         = -4;
    af.bitvector        = 0;
    af.procNo           = 0;
    af.flags            = AFF_PRI_MEDIUM;
    af.name             = "";
    if ( IS_NPC(victim) )
    af.appearance       = "";
    else
    af.appearance       = "%n %p, covered in a seething mass of tiny spiders.";
    affect_to_char( victim, &af );

    for ( try = 0; try < level / 10; ++try ) 
    {
        if ( saves_spell(level, victim, DAM_POISON) )
            continue;

        af.type         = gsn_poison;
        af.level        = level / 2;
        af.duration     = level / 10;
        af.location     = APPLY_STR;
        af.bitvector    = AFF_POISON;
        af.procNo       = affect_proc_lookup( "poison" );
        af.flags        = AFF_CONT_UPDATE|AFF_PRI_LOW;
        if ( IS_NPC(victim) )
        af.appearance   = "";
        else
        af.appearance   = "%n %p, a sickly green tinge coloring %s skin.";
        affect_join( victim, &af );
        
        send_to_char( "A spider sinks its fangs into you!", victim );
        act( "$N suddenly looks very ill as a spider sinks its fangs into $M.",
             ch, NULL, victim, TO_ROOM );
        act( "$N suddenly looks very ill as a spider sinks its fangs into $M.",
             ch, NULL, victim, TO_CHAR );
    }
}


void spell_spider_familier( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *spider;
    
    if ( ch->pet )
    {
        send_to_char( "You already have a pet.\n\r", ch );
        return;
    }
    
    send_to_char( "A large hairy black spider crawls up your neck, coming to rest on top of your head!\n\r", ch );
    act( "A large hairy black spider crawls up $ns neck, coming to rest on top of $s head!",
         ch, NULL, NULL, TO_ROOM );

    spider = create_mobile( get_mob_index(MOB_VNUM_SPIDER_FAMILIER) );
    SET_BIT( spider->act, ACT_PET|ACT_FAMILIER );
    SET_BIT( spider->affected_by, AFF_CHARM );
    spider->comm = COMM_NOTELL|COMM_NOSHOUT|COMM_NOCHANNELS;
    level_familier( spider, ch->level );
    char_to_room( spider, ch->in_room );
    add_follower( spider, ch );
    spider->leader = ch;
    ch->pet = spider;
}


/* =================================================================== */


void spell_animal_form( int sn, int level, CHAR_DATA *ch, void *vo, int target )
{
    CHAR_DATA *victim = (CHAR_DATA *) vo;
    bool replaceHits;
    AFFECT_DATA af;
    
    if ( ch->polymorphed )
    {
        send_to_char( "You can't change form again.\n\r", ch );
        return;
    }
    
    if ( !IS_NPC(victim)
    ||   !IS_SET(CHAR_FORM(victim), FORM_ANIMAL) )
    {
        send_to_char( "They aren't an animal.\n\r", ch );
        return;
    }
    
    if ( victim->level > level )
    {
        send_to_char( "Their power exceeds your own.\n\r", ch );
        return;
    }
    
    act( "With a hideously tortured scream $n transforms $mself into $N!",
         ch, NULL, victim, TO_ROOM );
    send_to_char( "An agonizing pain wracks your body as you change shape!\n\r", ch );

    replaceHits                 = ch->hit * calc_mob_hits( victim->level ) / ch->max_hit > ch->hit;
    if ( replaceHits )
        ch->hit                 = ch->hit * calc_mob_hits( victim->level ) / ch->max_hit;

    af.where                    = TO_POLYMORPH;
    af.type                     = sn;
    af.level                    = victim->level;
    af.duration                 = level / 10;
    af.location                 = APPLY_NONE;
    af.modifier                 = victim->pIndexData->vnum;
    af.bitvector                = 0;
    af.procNo                   = affect_proc_lookup("polymorph");
    af.flags                    = AFF_PROC_FINISHED;
    af.name                     = "";
    af.appearance               = "";
    affect_to_char( ch, &af );
    
    af.procNo                   = 0;
    af.flags                    = 0;
    
    if ( IS_AFFECTED(victim, AFF_FLYING) )
    {
    af.where                    = TO_AFFECTS;
    af.modifier                 = 0;
    af.bitvector                = AFF_FLYING;
    affect_to_char( ch, &af );
    }

    af.where                    = TO_AFFECTS2;
    if ( replaceHits )
    {
    af.location                 = APPLY_HIT;
    af.modifier                 = calc_mob_hits( victim->level ) - ch->max_hit;
    }
    af.bitvector                = AFF2_NO_EQUIPMENT + AFF2_CANT_CAST + AFF2_HALF_XP;
    af.flags                    = AFF_INVIS;
    affect_to_char( ch, &af );
    
    remove_all_equipment( ch );
}

