/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#endif
#include "merc.h"
#include "recycle.h"


void remove_resets( AREA_DATA *area, char type, int arg1, int arg2 )
{
    RESET_DATA *current, *previous;
    
    current = previous = area->reset_first;
    
    while ( current )
    {
	if ( current->command == type )
	{
	    bool match = FALSE;
	    
	    switch ( type )
	    {
	    case 'M':
		if ( arg1 == current->arg1
		&&   (arg2 == 0 || arg2 == current->arg3) )
		{
		    while ( current->next )
		    {
			if ( current->next->command == 'E'
			||   current->next->command == 'G' )
			{
			    RESET_DATA *temp;
			    
			    temp		= current->next;
			    current->next	= temp->next;
			    
			    free_reset_data( temp );
			}
			else
			    break;
		    }
		    
		    match = TRUE;
		}
		break;
	    case 'O':
		if ( arg1 == current->arg1
		&&   arg2 == current->arg3 )
		    match = TRUE;
		break;
	    case 'D':
		if ( arg1 == current->arg1
		&&   arg2 == current->arg2 )
		    match = TRUE;
		break;
	    }
	    
	    if ( match )
	    {
		if ( previous == current )
		{
		    area->reset_first = current->next;

		    free_reset_data( current );

		    previous = current = area->reset_first;
		}
		else
		{
		    previous->next = current->next;

		    free_reset_data( current );

		    current = previous;
		}
		    
		continue;
	    }
	}
	
	previous = current;
	current  = current->next;
    }
}


void remove_reset( AREA_DATA *area, RESET_DATA *pReset )
{
    RESET_DATA *previous;
    
    if ( pReset->command == 'M' )
    {
	while ( pReset->next )
	{
	   if ( pReset->next->command == 'E'
	   ||   pReset->next->command == 'G' )
	   {
		RESET_DATA *temp;

		temp		= pReset->next;
		pReset->next	= temp->next;

		free_reset_data( temp );
	    }
	    else
		break;
	}
    }
    
    if ( pReset == area->reset_first )
    {
	area->reset_first = pReset->next;
	if ( area->reset_last == pReset )
	    area->reset_last = NULL;
    }
    else
    {
	for ( previous = area->reset_first;
	      previous != NULL;
	      previous = previous->next )
	{
	    if ( previous->next == pReset )
	    {
		previous->next = pReset->next;
		break;
	    }
	}
    }

    free_reset_data( pReset );
}


RESET_DATA *find_reset( AREA_DATA *area, char type, int arg1, int arg2 )
{
    RESET_DATA *pReset;
    
    for ( pReset = area->reset_first; pReset != NULL; pReset = pReset->next )
    {
	if ( type == pReset->command )
	{
	    switch ( type )
	    {
	    case 'M':
		if ( arg1 == pReset->arg1
		&&   arg2 == pReset->arg3 )
		    return pReset;
		break;
	    case 'O':
		if ( arg1 == pReset->arg1
		&&   arg2 == pReset->arg3 )
		    return pReset;
		break;
	    case 'D':
		if ( arg1 == pReset->arg1
		&&   arg2 == pReset->arg2 )
		    return pReset;
		break;
	    }
	}
    }
    
    return NULL;
}


RESET_DATA *create_reset( AREA_DATA *area, RESET_DATA *after )
{
    RESET_DATA *pReset;
    
    pReset = new_reset_data();
    
    if ( after )
    {
	pReset->next	= after->next;
	after->next	= pReset;
	
	if ( area->reset_last == after )
	    area->reset_last = pReset;
    }
    else
    {
        pReset->next = area->reset_first;
	area->reset_first = pReset;

	if ( area->reset_last == NULL )
	    area->reset_last = pReset;
    }
    
    return pReset;
}
