/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#endif
#include "olc.h"
#include "merc.h"
#include "db.h"
#include "recycle.h"
#include "lookup.h"
#include "tables.h"


/* Globals. */
extern char *const dir_name[];


/* Local function prototypes. */
int	exit_locks		args( ( EXIT_DATA *pExit ) );
char	*print_masked_flags	args( ( int flags, int mask ) );
char	*print_word		args( ( char *word ) );
void	fwrite_remove_mob_flag	args( ( FILE *fp, char *tag, 
					int flags, int standard ) );
void	fwrite_mobiles		args( ( FILE *fp, AREA_DATA *area ) );
void	fwrite_objects		args( ( FILE *fp, AREA_DATA *area ) );
void	fwrite_rooms		args( ( FILE *fp, AREA_DATA *area ) );
void	fwrite_resets		args( ( FILE *fp, AREA_DATA *area ) );
void	fwrite_shops		args( ( FILE *fp, AREA_DATA *area ) );
void	fwrite_specials		args( ( FILE *fp, AREA_DATA *area ) );
void	fwrite_programs		args( ( FILE *fp, AREA_DATA *area ) );


int exit_locks( EXIT_DATA *pExit )
{
    if ( IS_SET(pExit->exit_info, EX_ISDOOR)
    &&   IS_SET(pExit->exit_info, EX_NOPASS)
    &&   IS_SET(pExit->exit_info, EX_PICKPROOF)
    &&   IS_SET(pExit->exit_info, EX_NGBACK) )
	return 8;
    if ( IS_SET(pExit->exit_info, EX_ISDOOR)
    &&   IS_SET(pExit->exit_info, EX_PICKPROOF)
    &&   IS_SET(pExit->exit_info, EX_NGBACK) )
	return 7;
    if ( IS_SET(pExit->exit_info, EX_ISDOOR)
    &&   IS_SET(pExit->exit_info, EX_NOPASS)
    &&   IS_SET(pExit->exit_info, EX_NGBACK) )
	return 6;
    if ( IS_SET(pExit->exit_info, EX_ISDOOR)
    &&   IS_SET(pExit->exit_info, EX_NGBACK) )
	return 5;
    if ( IS_SET(pExit->exit_info, EX_ISDOOR)
    &&   IS_SET(pExit->exit_info, EX_NOPASS)
    &&   IS_SET(pExit->exit_info, EX_PICKPROOF) )
	return 4;
    else if ( IS_SET(pExit->exit_info, EX_ISDOOR)
    &&   IS_SET(pExit->exit_info, EX_NOPASS) )
	return 3;
    else if ( IS_SET(pExit->exit_info, EX_ISDOOR)
    &&   IS_SET(pExit->exit_info, EX_PICKPROOF) )
	return 2;
    else if ( IS_SET(pExit->exit_info, EX_ISDOOR) )
	return 1;
    else
	return 0;
}


void fwrite_description( FILE *fp, const char *description )
{
    char buf[MAX_STRING_LENGTH];
    char *dest = buf;
    for ( ; *description; ++description )
    {
        if ( *description == '\r' )
            continue;

        *dest = *description;
        ++dest;
    }
    *dest = '\0';

    fprintf( fp, "%s~\n", buf );
}


/* Build a flag string of all bits in flags that are not in mask. */
char *print_masked_flags( int flags, int mask )
{
    int count, pos = 0;
    static char buf[52];


    for (count = 0; count < 32;  count++)
    {
        if (IS_SET(flags,1<<count) && !IS_SET(mask,1<<count))
        {
            if (count < 26)
                buf[pos] = 'A' + count;
            else
                buf[pos] = 'a' + (count - 26);
            pos++;
        }
    }

    if (pos == 0)
    {
        buf[pos] = '0';
        pos++;
    }

    buf[pos] = '\0';

    return buf;
}


char *print_word( char *word )
{
   if ( strstr(word, " ") )
   {
	static char buf[ MAX_STRING_LENGTH ];
	
	strcpy( buf, "'" );
	strcat( buf, word );
	strcat( buf, "'" );

	return buf;
   }
   else
	return word;
}


void fwrite_remove_mob_flag( FILE *fp, char *tag, int flags, int standard )
{
    int removed_bits;
    
    removed_bits = (flags ^ standard) & standard;
    
    if ( removed_bits )
	fprintf( fp, "F %s %s\n", tag, print_flags(removed_bits) );
}


void fwrite_area( AREA_DATA *area )
{
    char strSave[MAX_STRING_LENGTH];
    FILE *fp;
    
    REMOVE_BIT( area->flags, AREA_DIRTY );

    fclose( fpReserve );
    if ( (fp = fopen(WORKING_TEMP, "w")) == NULL )
    {
	bug( "fwrite_area: fopen.", 0 );
	perror( area->file_name );

	fpReserve = fopen( NULL_FILE, "r" );
	return;
    }
    
    /* Write the area header. */
    fprintf( fp, "#OLCAREA %d\n", OLC_AREA_VERSION );
    fprintf( fp, "%s~\n", area->file_name );
    fprintf( fp, "%s~\n", area->name );
    fprintf( fp, "%s~\n", area->credits );
    
    fprintf( fp, "%d %d %s\n", 
        area->min_vnum, 
        area->max_vnum,
        print_flags(area->flags) );
    fprintf( fp, "%s~\n", area->creator );
    fprintf( fp, "%s~\n\n\n", area->sound_track );
    
    /* Write out the other bits. */
    fwrite_mobiles( fp, area );
    fwrite_objects( fp, area );
    fwrite_rooms( fp, area );
    fwrite_resets( fp, area );
    fwrite_shops( fp, area );
    fwrite_specials( fp, area );
    fwrite_programs( fp, area );
    
    fprintf( fp, "#$\n" );
    
    fclose( fp );
    sprintf( strSave, "%s%s", WORKING_AREAS, area->file_name );
    #if defined(_WIN32)
    remove( strSave );
    #endif
    rename( WORKING_TEMP, strSave );
    fpReserve = fopen( NULL_FILE, "r" );
    return;
}


void fwrite_mobiles( FILE *fp, AREA_DATA *area )
{
    MOB_INDEX_DATA *pMobIndex;
    bool first = TRUE;
    int flags;
    int vnum;

    for ( vnum = area->min_vnum; vnum <= area->max_vnum; vnum++ )
    {
	MPROG_LIST *pMprog;

	pMobIndex = get_mob_index( vnum );
	if ( pMobIndex == NULL )
	    continue;
	
	if ( first )
	{
	    fprintf( fp, "#OLCMOBILES %d\n\n", OLC_MOBILE_VERSION );
	    
	    first = FALSE;
	}
	
	fprintf( fp, "#%d\n", vnum );
	fprintf( fp, "%s~\n", pMobIndex->player_name );
	fprintf( fp, "%s~\n", pMobIndex->short_descr );
	fprintf( fp, "%s~\n", pMobIndex->long_descr );
        fwrite_description( fp, pMobIndex->description );
	//fprintf( fp, "%s~\n", pMobIndex->description );
	fprintf( fp, "%s~\n", race_table[pMobIndex->race].name );
	
	flags = pMobIndex->act;
	REMOVE_BIT( flags, ACT_IS_NPC );
	fprintf( fp, "%s ",
	    print_masked_flags(flags, 
	                       race_table[pMobIndex->race].act) );
	fprintf( fp, "%s ", print_flags(pMobIndex->aggression) );
	fprintf( fp, "%s ",
	    print_masked_flags(pMobIndex->affected_by, 
	                       race_table[pMobIndex->race].aff) );
	fprintf( fp, "%s %d %d\n",
	    print_masked_flags(pMobIndex->affected_by2,
	                       race_table[pMobIndex->race].aff2),
	    pMobIndex->alignment,
	    pMobIndex->group );
	
	fprintf( fp, "%d %d %dd%d+%d %dd%d+%d %dd%d+%d %s\n",
	    pMobIndex->level,
	    pMobIndex->hitroll,
	    pMobIndex->hit[DICE_NUMBER],
	    pMobIndex->hit[DICE_TYPE],
	    pMobIndex->hit[DICE_BONUS],
	    pMobIndex->mana[DICE_NUMBER],
	    pMobIndex->mana[DICE_TYPE],
	    pMobIndex->mana[DICE_BONUS],
	    pMobIndex->damage[DICE_NUMBER],
	    pMobIndex->damage[DICE_TYPE],
	    pMobIndex->damage[DICE_BONUS],
	    attack_table[pMobIndex->dam_type].name );

	fprintf( fp, "%d %d %d %d\n",
	    pMobIndex->ac[AC_PIERCE]	/ 10,
	    pMobIndex->ac[AC_BASH]	/ 10,
	    pMobIndex->ac[AC_SLASH]	/ 10,
	    pMobIndex->ac[AC_EXOTIC] 	/ 10 );

	fprintf( fp, "%s ",
	    print_masked_flags(pMobIndex->off_flags, 
	                       race_table[pMobIndex->race].off) );
	fprintf( fp, "%s ",
	    print_masked_flags(pMobIndex->imm_flags, 
	                       race_table[pMobIndex->race].imm) );
	fprintf( fp, "%s ",
	    print_masked_flags(pMobIndex->res_flags, 
	                       race_table[pMobIndex->race].res) );
	fprintf( fp, "%s\n",
	    print_masked_flags(pMobIndex->vuln_flags, 
	                       race_table[pMobIndex->race].vuln) );

	fprintf( fp, "%s %s %s %ld\n",
	    position_table[pMobIndex->start_pos].short_name,
	    position_table[pMobIndex->default_pos].short_name,
	    sex_table[pMobIndex->sex].name,
	    pMobIndex->wealth );
	
	fprintf( fp, "%s ",
	    print_masked_flags(pMobIndex->form, 
	                       race_table[pMobIndex->race].form) );
	fprintf( fp, "%s %s %s\n",
	    print_masked_flags(pMobIndex->parts, 
	                       race_table[pMobIndex->race].parts),
	    size_table[pMobIndex->size].name,
	    print_word(pMobIndex->material) );
	
	fwrite_remove_mob_flag( 
	    fp, "act", pMobIndex->act,
	    race_table[pMobIndex->race].act );
	fwrite_remove_mob_flag( 
	    fp, "aff", pMobIndex->affected_by,
	    race_table[pMobIndex->race].aff );
	fwrite_remove_mob_flag(
	    fp, "aff2", pMobIndex->affected_by2,
	    race_table[pMobIndex->race].aff2 );
	fwrite_remove_mob_flag( 
	    fp, "off", pMobIndex->off_flags,
	    race_table[pMobIndex->race].off );
	fwrite_remove_mob_flag( 
	    fp, "imm", pMobIndex->imm_flags,
	    race_table[pMobIndex->race].imm );
	fwrite_remove_mob_flag( 
	    fp, "res", pMobIndex->res_flags,
	    race_table[pMobIndex->race].res );
	fwrite_remove_mob_flag( 
	    fp, "vul", pMobIndex->vuln_flags,
	    race_table[pMobIndex->race].vuln );
	fwrite_remove_mob_flag( 
	    fp, "for", pMobIndex->form,
	    race_table[pMobIndex->race].form );
	fwrite_remove_mob_flag( 
	    fp, "par", pMobIndex->parts,
	    race_table[pMobIndex->race].parts );

	for ( pMprog = pMobIndex->mprogs; pMprog; pMprog = pMprog->next )
	{
	    fprintf( fp, "M %s %d %s~\n",
	        mprog_type_to_name(pMprog->trig_type),
	        pMprog->vnum,
	        pMprog->trig_phrase );
	}
    }
    
    if ( !first )
	fprintf( fp, "#0\n\n\n" );
}


void fwrite_objects( FILE *fp, AREA_DATA *area )
{
    OBJ_INDEX_DATA *pObjIndex;
    USE_MSG_DATA *use_msg;
    EXTRA_DESCR_DATA *ed;
    
    bool first = TRUE;
    AFFECT_DATA *paf;
    int vnum;

    for ( vnum = area->min_vnum; vnum <= area->max_vnum; vnum++ )
    {
	MPROG_LIST *pMprog;

	pObjIndex = get_obj_index( vnum );
	if ( pObjIndex == NULL )
	    continue;
	
	if ( first )
	{
	    fprintf( fp, "#OLCOBJECTS %d\n\n", OLC_OBJECT_VERSION );
	    
	    first = FALSE;
	}
	
	fprintf( fp, "#%d\n", vnum );
	fprintf( fp, "%s~\n", pObjIndex->name );
	fprintf( fp, "%s~\n", pObjIndex->short_descr );
        fprintf( fp, "%s~\n", pObjIndex->description );
	fprintf( fp, "%s~\n", pObjIndex->material );
	
	fprintf( fp, "%s %s ",
	    item_name(pObjIndex->item_type),
	    print_flags(pObjIndex->extra_flags) );
	fprintf( fp, "%s ",
	    print_flags(pObjIndex->wear_flags) );
	fprintf( fp, "%s ",
	    print_flags(pObjIndex->fit_flags) );
	fprintf( fp, "%s\n",
	    print_flags(pObjIndex->restrictions) );
	
	switch ( pObjIndex->item_type )
	{
	    case ITEM_WEAPON:
		fprintf( fp, "%s %d %d %s %s\n",
		    weapon_name(pObjIndex->value[0]),
		    pObjIndex->value[1],
		    pObjIndex->value[2],
		    attack_table[pObjIndex->value[3]].name,
		    print_flags(pObjIndex->value[4]) );
		 break;
	    case ITEM_ARMOR:
	    case ITEM_LIGHT:
	    case ITEM_MONEY:
		fprintf( fp, "%d %d %d %d %d\n",
		    pObjIndex->value[0],
		    pObjIndex->value[1],
		    pObjIndex->value[2],
		    pObjIndex->value[3],
		    pObjIndex->value[4] );
		break;
	    case ITEM_QUIVER:
		fprintf( fp, "%d %d %d 0 0\n",
		    pObjIndex->value[0],
		    pObjIndex->value[1],
		    pObjIndex->value[2] );
		break;
	    case ITEM_ARROW:
		fprintf( fp, "0 %d %d 0 0\n",
		    pObjIndex->value[1],
		    pObjIndex->value[2] );
		break;
	    case ITEM_FOOD:
		fprintf( fp, "%d %d %d %s %d\n",
		    pObjIndex->value[0],
		    pObjIndex->value[1],
		    pObjIndex->value[2],
		    print_flags(pObjIndex->value[3]),
		    pObjIndex->value[4] );
		break;
	    case ITEM_PORTAL:
		fprintf( fp, "%d %s ",
		    pObjIndex->value[0],
		    print_flags(pObjIndex->value[1]) );
		fprintf( fp, "%s %d %d\n",
		    print_flags(pObjIndex->value[2]),
		    pObjIndex->value[3],
		    pObjIndex->value[4] );
		break;
	    case ITEM_CONTAINER:
		fprintf( fp, "%d %s %d %d %d\n",
		    pObjIndex->value[0],
		    print_flags(pObjIndex->value[1]),
		    pObjIndex->value[2],
		    pObjIndex->value[3],
		    pObjIndex->value[4] );
		break;
	    case ITEM_DRINK_CON:
	    case ITEM_FOUNTAIN:
		fprintf( fp, "%d %d '%s' %d %d\n",
		    pObjIndex->value[0],
		    pObjIndex->value[1],
		    liq_table[pObjIndex->value[2]].liq_name,
		    pObjIndex->value[3],
		    pObjIndex->value[4] );
		break;
	    case ITEM_WAND:
	    case ITEM_STAFF:
		fprintf( fp, "%d %d %d '%s' %d\n",
		    pObjIndex->value[0],
		    pObjIndex->value[1],
		    pObjIndex->value[2],
		    pObjIndex->value[3] >= 0
		    ? skill_table[pObjIndex->value[3]].name
		    : "",
		    pObjIndex->value[4] );
		break;
	    case ITEM_POTION:
	    case ITEM_PILL:
	    case ITEM_SCROLL:
		fprintf( fp, "%d '%s' '%s' '%s' '%s'\n",
		    pObjIndex->value[0],
		    pObjIndex->value[1] >= 0
		    ? skill_table[pObjIndex->value[1]].name
		    : "",
		    pObjIndex->value[2] >= 0
		    ? skill_table[pObjIndex->value[2]].name
		    : "",
		    pObjIndex->value[3] >= 0
		    ? skill_table[pObjIndex->value[3]].name
		    : "",
		    pObjIndex->value[4] >= 0
		    ? skill_table[pObjIndex->value[4]].name
		    : "" );
		break;
	    case ITEM_FURNITURE:
		fprintf( fp, "%d %d %s %d %d\n",
		    pObjIndex->value[0],
		    pObjIndex->value[1],
		    print_flags(pObjIndex->value[2]),
		    pObjIndex->value[3],
		    pObjIndex->value[4] );
		break;
	    default:
		fprintf( fp, "%s ",
		    print_flags(pObjIndex->value[0]) );
		fprintf( fp, "%s ",
		    print_flags(pObjIndex->value[1]) );
		fprintf( fp, "%s ",
		    print_flags(pObjIndex->value[2]) );
		fprintf( fp, "%s ",
		    print_flags(pObjIndex->value[3]) );
		fprintf( fp, "%s\n",
		    print_flags(pObjIndex->value[4]) );
		break;
	}

	fprintf( fp, "%d %d %d P\n",	/* Perfect condition only supported. */
	    pObjIndex->level,
	    pObjIndex->weight,
	    pObjIndex->cost );
	
	for ( paf = pObjIndex->affected; paf != NULL; paf = paf->next )
	{
	    if ( paf->bitvector )
	    {
		char where;
		
		switch ( paf->where )
		{
		case TO_AFFECTS:	where = 'A';	break;
		case TO_AFFECTS2:	where = 'B';	break;
		case TO_IMMUNE:		where = 'I';	break;
		case TO_RESIST:		where = 'R';	break;
		case TO_VULN:		where = 'V';	break;
		default:
		    bug( "fwrite_objects: unknown affect where location.", 0 );
		    where = 'A';
		    break;
		}
		
		fprintf( fp, "F %c %d %d %s\n",
		    where,
		    paf->location,
		    paf->modifier,
		    print_flags(paf->bitvector) );
	    }
	    else
		fprintf( fp, "A %d %d\n", paf->location, paf->modifier );
	}
	
	for ( ed = pObjIndex->extra_descr; ed != NULL; ed = ed->next )
        {
	    fprintf( fp, "E\n%s~\n", ed->keyword );
            fwrite_description( fp, ed->description );
        }
/*	
 *	if ( pObjIndex->remove_self )
 *	    fprintf( fp, "R S %s\n", pObjIndex->remove_self );
 *	if ( pObjIndex->remove_room )
 *	    fprintf( fp, "R R %s\n", pObjIndex->remove_room );
 *	if ( pObjIndex->wear_self )
 *	    fprintf( fp, "W S %s\n", pObjIndex->wear_self );
 *	if ( pObjIndex->wear_room )
 *	    fprintf( fp, "W R %s\n", pObjIndex->wear_room );
 *	if ( pObjIndex->wear_loc )
 *	    fprintf( fp, "W L %s\n", pObjIndex->wear_loc );
 */
	for ( use_msg = pObjIndex->use_msgs; use_msg != NULL; use_msg = use_msg->next )
	{
	    fprintf( fp, "U %d ", use_msg->tag );
            fwrite_description( fp, use_msg->msg );
	}

	for ( pMprog = pObjIndex->mprogs; pMprog; pMprog = pMprog->next )
	{
	    fprintf( fp, "M %s %d %s~\n",
	        oprog_type_to_name(pMprog->trig_type),
	        pMprog->vnum,
	        pMprog->trig_phrase );
	}
    }
    
    if ( !first )
	fprintf( fp, "#0\n\n\n" );
}


void fwrite_rooms( FILE *fp, AREA_DATA *area )
{
    ROOM_INDEX_DATA *pRoomIndex;
    EXTRA_DESCR_DATA *ed;
    bool first = TRUE;
    int vnum;
    int door;

    for ( vnum = area->min_vnum; vnum <= area->max_vnum; vnum++ )
    {
	MPROG_LIST *pMprog;
        long flags;

	pRoomIndex = get_room_index( vnum );
	if ( pRoomIndex == NULL )
	    continue;
	
	if ( first )
	{
	    fprintf( fp, "#OLCROOMS %d\n\n", OLC_ROOM_VERSION );
	    
	    first = FALSE;
	}
	
	fprintf( fp, "#%d\n", vnum );
	fprintf( fp, "%s~\n", pRoomIndex->name );
        fwrite_description( fp, pRoomIndex->description );
	//fprintf( fp, "%s~\n", pRoomIndex->description );
	
        flags = pRoomIndex->room_flags & ~ROOM_EQUIPMENT_LOADED;
	fprintf( fp, "0 %s %d\n",
	    print_flags(flags),
	    pRoomIndex->sector_type );
	
	for ( door = 0; door <= 11; door++ )
	{
	    if ( pRoomIndex->old_exit[door] == NULL )
		continue;

	    fprintf( fp, "D%d\n", door );
            fwrite_description( fp, pRoomIndex->old_exit[door]->description );
            fprintf( fp, "%s~\n%d %d %d\n",
		pRoomIndex->old_exit[door]->keyword,
		exit_locks(pRoomIndex->old_exit[door]),
		pRoomIndex->old_exit[door]->key,
		pRoomIndex->old_exit[door]->u1.to_room
		 ? pRoomIndex->old_exit[door]->u1.to_room->vnum
		 : -1 );
	}
	
	for ( ed = pRoomIndex->extra_descr; ed != NULL; ed = ed->next )
        {
	    fprintf( fp, "E\n%s~\n", ed->keyword );
            fwrite_description( fp, ed->description );
        }
	    
	if ( pRoomIndex->heal_rate != 100 )
	    fprintf( fp, "H %d\n", pRoomIndex->heal_rate );
	if ( pRoomIndex->mana_rate != 100 )
	    fprintf( fp, "M %d\n", pRoomIndex->heal_rate );
	
	if ( pRoomIndex->clan )
	    fprintf( fp, "C\n%s~\n", clan_table[pRoomIndex->clan].name );
	if ( pRoomIndex->owner )
	    fprintf( fp, "O\n%s~\n", pRoomIndex->owner );

	for ( pMprog = pRoomIndex->mprogs; pMprog; pMprog = pMprog->next )
	{
	    fprintf( fp, "P %s %d %s~\n",
	        rprog_type_to_name(pMprog->trig_type),
	        pMprog->vnum,
	        pMprog->trig_phrase );
	}
	    
	fprintf( fp, "S\n" );
    }

    if ( !first )
	fprintf( fp, "#0\n\n\n" );
}


void fwrite_resets( FILE *fp, AREA_DATA *area )
{
    ROOM_INDEX_DATA *pRoomIndex	= NULL;
    MOB_INDEX_DATA *pMobIndex	= NULL;
    OBJ_INDEX_DATA *pObjIndex	= NULL;
    RESET_DATA *pReset;
    
    if ( area->reset_first == NULL )
	return;
	
    fprintf( fp, "#OLCRESETS %d\n\n", OLC_RESET_VERSION );
    
    for ( pReset = area->reset_first; pReset != NULL; pReset = pReset->next )
    {
	switch ( pReset->command ) 
	{
	case 'M':
	    pMobIndex = get_mob_index( pReset->arg1 );
	    if ( pMobIndex == NULL )
	    {
		bug( "fwrite_resets: bad mob vnum in reset (%d).", pReset->arg1 );
		continue;
	    }
	    
	    pRoomIndex = get_room_index( pReset->arg3 );
	    if ( pRoomIndex == NULL )
	    {
		bug( "fwrite_resets: bad room vnum in reset (%d).", pReset->arg3 );
		continue;
	    }

	    fprintf( fp, "M 0 %5d %2d %5d %2d \t* %s\n",
		pReset->arg1,
		pReset->arg2,
		pReset->arg3,
		pReset->arg4,
		pMobIndex->short_descr );
	    break;
	
	case 'O':
	    pObjIndex = get_obj_index( pReset->arg1 );
	    if ( pObjIndex == NULL )
	    {
		bug( "fwrite_resets: bad obj vnum in reset (%d).", pReset->arg1 );
		continue;
	    }

	    pRoomIndex = get_room_index( pReset->arg3 );
	    if ( pRoomIndex == NULL )
	    {
		bug( "fwrite_resets: bad room vnum in reset (%d).", pReset->arg3 );
		continue;
	    }

	    fprintf( fp, "O 0 %5d %2d %5d    \t* %s\n",
		pReset->arg1,
		pReset->arg2,
		pReset->arg3,
		pObjIndex->short_descr );
	    break;
	
	case 'P':
	    pObjIndex = get_obj_index( pReset->arg3 );
	    if ( pObjIndex == NULL )
	    {
		bug( "fwrite_resets: bad obj vnum in reset (%d).", pReset->arg3 );
		continue;
	    }

	    pObjIndex = get_obj_index( pReset->arg1 );
	    if ( pObjIndex == NULL )
	    {
		bug( "fwrite_resets: bad obj vnum in reset (%d).", pReset->arg1 );
		continue;
	    }

	    fprintf( fp, "P 1 %5d %2d %5d %2d \t*   %s\n",
		pReset->arg1,
		pReset->arg2,
		pReset->arg3,
		pReset->arg4,
		pObjIndex->short_descr );
	    break;
	
	case 'G':
	    pObjIndex = get_obj_index( pReset->arg1 );
	    if ( pObjIndex == NULL )
	    {
		bug( "fwrite_resets: bad obj vnum in reset (%d).", pReset->arg1 );
		continue;
	    }

	    fprintf( fp, "G 1 %5d %2d          \t*   %s\n",
		pReset->arg1,
		pReset->arg2,
		pObjIndex->short_descr );
	   break;
	   
	case 'E':
	    pObjIndex = get_obj_index( pReset->arg1 );
	    if ( pObjIndex == NULL )
	    {
		bug( "fwrite_resets: bad obj vnum in reset (%d).", pReset->arg1 );
		continue;
	    }
	    
	    fprintf( fp, "E 1 %5d %2d %5d    \t*   %s\n",
		pReset->arg1,
		pReset->arg2,
		pReset->arg3,
		pObjIndex->short_descr );
	    break;
	
	case 'D':
	    pRoomIndex = get_room_index( pReset->arg1 );
	    if ( pRoomIndex == NULL )
	    {
		bug( "fwrite_resets: bad room vnum in reset (%d).", pReset->arg1 );
		exit( 1 );
	    }

	    fprintf( fp, "D 0 %5d %2d %5d    \t* %s, %s\n",
		pReset->arg1,
		pReset->arg2,
		pReset->arg3,
		pRoomIndex->name,
		dir_name[pReset->arg2] );
	    break;

	case 'R':
	    fprintf( fp, "R 0 %5d %2d\n",
		pReset->arg1,
		pReset->arg2 );  
	    break;
	
	case 'T':
	    if ( IS_SET(pReset->arg3, TRAP_ROOM) )
		fprintf( fp, "T 0 %2d %2d %5d %2d %5d\n",
		    pReset->arg1,
		    pReset->arg2,
		    pReset->arg3,
		    pReset->arg4,
		    pReset->arg5 ); 
	    else
		fprintf( fp, "T 0 %2d %2d %5d %2d %5d\n",
		    pReset->arg1,
		    pReset->arg2,
		    pReset->arg3,
		    pReset->arg4,
		    pReset->arg5 ); 
	    break;
	default:
	    bug( "fwrite_resets: unknown reset command (%c).", pReset->command );
	}
    }
    
    fprintf( fp, "S\n\n\n" );
}


void fwrite_shops( FILE *fp, AREA_DATA *area )
{
    MOB_INDEX_DATA *pMobIndex;
    bool first = TRUE;
    int vnum;

    for ( vnum = area->min_vnum; vnum <= area->max_vnum; vnum++ )
    {
	pMobIndex = get_mob_index( vnum );
	if ( pMobIndex == NULL || pMobIndex->pShop == NULL )
	    continue;
	
	if ( first )
	{
	    fprintf( fp, "#OLCSHOPS %d\n\n", OLC_SHOP_VERSION );
	    
	    first = FALSE;
	}
	
	fprintf( fp, "%-5d %2d %2d %2d %2d %2d  %3d %3d  %2d %2d  * %s\n",
	    vnum,
	    pMobIndex->pShop->buy_type[0],
	    pMobIndex->pShop->buy_type[1],
	    pMobIndex->pShop->buy_type[2],
	    pMobIndex->pShop->buy_type[3],
	    pMobIndex->pShop->buy_type[4],
	    pMobIndex->pShop->profit_buy,
	    pMobIndex->pShop->profit_sell,
	    pMobIndex->pShop->open_hour,
	    pMobIndex->pShop->close_hour,
	    pMobIndex->short_descr );
    }
    
    if ( !first )
	fprintf( fp, "0\n\n\n" );
}


void fwrite_specials( FILE *fp, AREA_DATA *area )
{
    MOB_INDEX_DATA *pMobIndex;
    bool first = TRUE;
    int vnum;

    for ( vnum = area->min_vnum; vnum <= area->max_vnum; vnum++ )
    {
	pMobIndex = get_mob_index( vnum );
	if ( pMobIndex == NULL || pMobIndex->spec_fun == NULL )
	    continue;
	
	if ( first )
	{
	    fprintf( fp, "#OLCSPECIALS %d\n\n", OLC_SPECIAL_VERSION );
	    
	    first = FALSE;
	}
	
	fprintf( fp, "M %5d %-30s  * %s\n",
	    vnum,
	    spec_name(pMobIndex->spec_fun),
	    pMobIndex->short_descr );
    }
    
    if ( !first )
	fprintf( fp, "S\n\n\n" );
}


void fwrite_programs( FILE *fp, AREA_DATA *area )
{
    MPROG_CODE *pProgram;
    bool first = TRUE;
    int vnum;

    for ( vnum = area->min_vnum; vnum <= area->max_vnum; vnum++ )
    {
	pProgram = get_mprog_index( vnum );
	if ( pProgram == NULL )
	    continue;
	
	if ( first )
	{
	    fprintf( fp, "#MOBPROGS\n\n" );
	    
	    first = FALSE;
	}
	
	fprintf( fp, "#%d\n", vnum );
        fwrite_description( fp, pProgram->code );
    }
    
    if ( !first )
	fprintf( fp, "#0\n\n\n" );
}

