/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "merc.h"
#include "color.h"
#include "magic.h"
#include "lookup.h"
#include "recycle.h"

/* command procedures needed */
DECLARE_DO_FUN(do_groups	);
DECLARE_DO_FUN(do_help		);
DECLARE_DO_FUN(do_say		);


/* Local function prototypes. */
bool group_usable( CHAR_DATA *ch, int gn );
bool skill_usable( CHAR_DATA *ch, int sn );


/* used to get new skills */
void do_gain(CHAR_DATA *ch, char *argument)
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *trainer;
    int gn = 0, sn = 0, diffrn = 0;

    if (IS_NPC(ch))
	return;

    /* find a trainer */
    for ( trainer = ch->in_room->people; 
	  trainer != NULL; 
	  trainer = trainer->next_in_room)
	if (IS_NPC(trainer) && IS_SET(trainer->act,ACT_GAIN))
	    break;

    if (trainer == NULL || !can_see(ch,trainer))
    {
	send_to_char("You can't do that here.\n\r",ch);
	return;
    }

    one_argument(argument,arg);

    if (arg[0] == '\0')
    {
	do_say(trainer,"Pardon me?");
	return;
    }

    if (!str_prefix(arg,"list"))
    {
	int col;
	
	col = 0;
	
	if ( IS_SET(class_table[ch->class].flags, CLASS_NOCUST) )
	{
	    do_say( trainer, "You cannot learn anything from me." );
	    return;
	}

	sprintf(buf, "%-19s %4s %-19s %4s %-19s %4s\n\r",
	             "group","cost","group","cost","group","cost");
	send_to_char(buf,ch);

	for (gn = 0; gn < MAX_GROUP; gn++)
	{
	    if (group_table[gn].name == NULL)
		break;

	    if (!ch->pcdata->group_known[gn]
	    &&  group_table[gn].rating[ch->class] > 0
	    &&  group_table[gn].gainable
	    &&  group_usable(ch, gn) )
	    {
		sprintf(buf,"%-19s %4d ",
		    group_table[gn].name,group_table[gn].rating[ch->class]);
		send_to_char(buf,ch);
		if (++col % 3 == 0)
		    send_to_char("\n\r",ch);
	    }
	}
	if (col % 3 != 0)
	    send_to_char("\n\r",ch);
	
	send_to_char("\n\r",ch);		

	col = 0;

        sprintf(buf, "%-19s %4s %-19s %4s %-19s %4s\n\r",
                     "skill","cost","skill","cost","skill","cost");
        send_to_char(buf,ch);
 
        for (sn = 0; sn < max_player_skill; sn++)
        {
            if (skill_table[sn].name == NULL)
                break;
 
            if (!ch->pcdata->learned[sn]
            &&  skill_table[sn].rating[ch->class] > 0
	    &&  skill_table[sn].spell_fun == spell_null
	    &&  skill_table[sn].gainable
	    &&  skill_usable(ch, sn) )
            {
                sprintf(buf,"%-19s %4d ",
                    skill_table[sn].name,skill_table[sn].rating[ch->class]);
                send_to_char(buf,ch);
                if (++col % 3 == 0)
                    send_to_char("\n\r",ch);
            }
        }
        if (col % 3 != 0)
            send_to_char("\n\r",ch);

	send_to_char("\n\r",ch);		
        
	col = 0;
        
	sprintf( buf, "%-31s%4s   %-31s%4s\n\r", 
	    "style/specialization", "cost",
	    "style/specialization", "cost" );
        
	send_to_char( buf, ch );

	for ( sn = 0; sn < MAX_STYLES; sn++ )
	{
	    if ( style_table[sn].name == NULL )
		break;

	    if ( !is_specialized(ch, sn)
	    &&   style_table[sn].rating[ch->class] > 0 )
	    {
		sprintf( buf, "%-31s%2d     ",
		   style_table[sn].name, style_table[sn].rating[ch->class] );
		send_to_char( buf, ch );
		if ( ++col % 2 == 0 )
		    send_to_char( "\n\r", ch ); 
	    }
	}
	
	if ( col % 2 != 0 )
	    send_to_char( "\n\r", ch );
	    
	return;
    }

    if (!str_prefix(arg,"convert"))
    {
	if (ch->practice < 10)
	{
	    act("$N tells you 'You are not yet ready.'",
		ch,NULL,trainer,TO_CHAR);
	    return;
	}

	act("$N helps you apply your practice to training",
		ch,NULL,trainer,TO_CHAR);
	ch->practice -= 10;
	ch->train +=1 ;
	return;
    }

    if (!str_prefix(arg,"points"))
    {
	if (ch->train < 2)
	{
	    act("$N tells you 'You are not yet ready.'",
		ch,NULL,trainer,TO_CHAR);
	    return;
	}

	if (ch->pcdata->points <= 40)
	{
	    act("$N tells you 'There would be no point in that.'",
		ch,NULL,trainer,TO_CHAR);
	    return;
	}

	act("$N trains you, and you feel more at ease with your skills.",
	    ch,NULL,trainer,TO_CHAR);

	diffrn = (ch->exp - (exp_per_level(ch, ch->pcdata->points) * ch->level));
	ch->train -= 2;
	ch->pcdata->points -= 1;
	ch->exp = ((exp_per_level(ch,ch->pcdata->points) * ch->level) + diffrn);
	return;
    }

    if ( IS_SET(class_table[ch->class].flags, CLASS_NOCUST) )
    {
	do_say( trainer, "You cannot learn that from me." );
	return;
    }

    /* else add a group/skill */
    gn = group_lookup(argument);
    if (gn > 0)
    {
	if (ch->pcdata->group_known[gn])
	{
	    act("$N tells you 'You already know that group!'",
		ch,NULL,trainer,TO_CHAR);
	    return;
	}

	if (group_table[gn].rating[ch->class] <= 0
        || !group_usable(ch, gn) )
	{
	    act("$N tells you 'That group is beyond your powers.'",
		ch,NULL,trainer,TO_CHAR);
	    return;
	}
	
	if ( !group_table[gn].gainable )
	{
	    act( "$N tells you 'I cannot teach that to you.'",
	        ch, NULL, trainer, TO_CHAR );
	    return;
	}

	if (ch->train < group_table[gn].rating[ch->class])
	{
	    act("$N tells you 'You are not yet ready for that group.'",
		ch,NULL,trainer,TO_CHAR);
	    return;
	}

	/* add the group */
	gn_add(ch,gn);
	act("$N trains you in the art of $t",
	    ch,group_table[gn].name,trainer,TO_CHAR);
	ch->train -= group_table[gn].rating[ch->class];
	return;
    }

    sn = skill_lookup(argument);
    if (sn > -1)
    {
	if (skill_table[sn].spell_fun != spell_null)
	{
	    act("$N tells you 'You must learn the full group.'",
		ch,NULL,trainer,TO_CHAR);
	    return;
	}
	    
        if (ch->pcdata->learned[sn])
        {
            act("$N tells you 'You already know that skill!'",
                ch,NULL,trainer,TO_CHAR);
            return;
        }
 
        if (skill_table[sn].rating[ch->class] <= 0
        ||  !skill_usable(ch, sn) )
        {
            act("$N tells you 'That skill is beyond your powers.'",
                ch,NULL,trainer,TO_CHAR);
            return;
        }
 
 	if ( !skill_table[sn].gainable )
	{
	    act( "$N tells you 'I cannot teach you that skill.'",
	        ch, NULL, trainer, TO_CHAR );
	    return;
	}


        if (ch->train < skill_table[sn].rating[ch->class] )
        {
            act("$N tells you 'You are not yet ready for that skill.'",
                ch,NULL,trainer,TO_CHAR);
            return;
        }
 
        /* add the skill */
	ch->pcdata->learned[sn] = 1;
        act("$N trains you in the art of $t.",
            ch,skill_table[sn].name,trainer,TO_CHAR);
        ch->train -= skill_table[sn].rating[ch->class];
        return;
    }

    sn = style_lookup( argument );
    if ( sn > -1 )
    {   
        if ( is_specialized(ch, sn) )
        {
            act("$N tells you 'You cannot learn that!'",
                ch, NULL, trainer, TO_CHAR );
            return;
        }
 
        if ( style_table[sn].rating[ch->class] <= 0 )
        {
            act("$N tells you 'That knowledge is beyond you.'",
                ch, NULL, trainer, TO_CHAR );
            return;
        }

        if ( ch->train < style_table[sn].rating[ch->class] )
        {
            act("$N tells you 'You are not yet ready to learn that.'",
                ch, NULL, trainer, TO_CHAR );
            return;
        }
        
        if ( !IS_NULLSTR(style_table[sn].prereq)
        &&   !ch->pcdata->learned[skill_lookup(style_table[sn].prereq)] )
        {
	    act( "$N tells you 'You need to learn $t first.'",
	        ch, style_table[sn].prereq, trainer, TO_CHAR );
	    return;
        }
 
        /* add the skill */
	ch->pcdata->styles[sn] = 1;
        act( "$N trains you in the art of $t.",
            ch, style_table[sn].name, trainer, TO_CHAR );
        ch->train -= style_table[sn].rating[ch->class];
        return;
    }

    act("$N tells you 'I do not understand...'",ch,NULL,trainer,TO_CHAR);
}
    



/* RT spells and skills show the players spells (or skills) */

void do_spells(CHAR_DATA *ch, char *argument)
{
    int sn, level, min_lev = 1, max_lev = LEVEL_HERO, mana;
    char buf[MAX_STRING_LENGTH],  buf2[MAX_STRING_LENGTH];
#if defined(macintosh)
    static char spell_list[LEVEL_HERO + 1][MAX_STRING_LENGTH];
#else
    char spell_list[LEVEL_HERO + 1][MAX_STRING_LENGTH];
#endif
    char spell_columns[LEVEL_HERO + 1];
    bool fAll = FALSE, found = FALSE;
    char arg[MAX_INPUT_LENGTH];
    bool weak_focus = FALSE;
    OBJ_DATA *focus = NULL;
    BUFFER *buffer;
 
    if ( IS_NPC(ch) )
      return;
      
    if ( ch->level < LEVEL_IMMORTAL
    &&   !IS_SET(class_table[ch->class].flags, CLASS_NEEDS_SYMBOL) )
    {
	OBJ_DATA *obj;

	for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
	{
	    if ( obj->item_type == ITEM_FOCUS
	    &&   obj->wear_loc != WEAR_NONE )
	    {
		if ( focus == NULL
		||   focus->value[0] < obj->value[0] )
		    focus = obj;
	    }
	}
    }

    if (argument[0] != '\0')
    {
	fAll = TRUE;

	if (str_prefix(argument,"all"))
	{
	    argument = one_argument(argument,arg);
	    if (!is_number(arg))
	    {
		send_to_char("Arguments must be numerical or all.\n\r",ch);
		return;
	    }
	    max_lev = atoi(arg);

	    if (max_lev < 1 || max_lev > LEVEL_HERO)
	    {
		sprintf(buf,"Levels must be between 1 and %d.\n\r",LEVEL_HERO);
		send_to_char(buf,ch);
		return;
	    }

	    if (argument[0] != '\0')
	    {
		argument = one_argument(argument,arg);
		if (!is_number(arg))
		{
		    send_to_char("Arguments must be numerical or all.\n\r",ch);
		    return;
		}
		min_lev = max_lev;
		max_lev = atoi(arg);

		if (max_lev < 1 || max_lev > LEVEL_HERO)
		{
		    sprintf(buf,
			"Levels must be between 1 and %d.\n\r",LEVEL_HERO);
		    send_to_char(buf,ch);
		    return;
		}

		if (min_lev > max_lev)
		{
		    send_to_char("That would be silly.\n\r",ch);
		    return;
		}
	    }
	}
    }

    /* initialize data */
    for (level = 0; level < LEVEL_HERO + 1; level++)
    {
        spell_columns[level] = 0;
        spell_list[level][0] = '\0';
    }
 
    for (sn = 0; sn < max_player_skill; sn++)
    {
        if (skill_table[sn].name == NULL )
	    break;

	if ( (level = skill_level(ch, sn)) < LEVEL_HERO + 1
	&&   (fAll || level <= ch->level)
	&&   level >= min_lev && level <= max_lev
	&&   skill_table[sn].spell_fun != spell_null
	&&   ch->pcdata->learned[sn] > 0 )
        {
	    found = TRUE;

	    if ( ch->level < level )
		sprintf( buf, "%-18.18s n/a                   ", skill_table[sn].name );
	    else
	    {
                char buf3[MAX_STRING_LENGTH];
                if ( ch->pcdata->skill_mods[sn] )
                    sprintf( buf3, "(%s%3d%%)",
                        ch->pcdata->skill_mods[sn] > 0 ? "+" : "-",
                        ch->pcdata->skill_mods[sn] > 0
                         ? ch->pcdata->skill_mods[sn]
                         : ch->pcdata->skill_mods[sn] * -1 );
                else
                    strcpy( buf3, "      " );

		mana = UMAX( skill_table[sn].min_mana,
		    100 / (2 + ch->level - level) );
		    
		if ( !IS_SET(class_table[ch->class].flags, CLASS_NEEDS_SYMBOL)
		&&   (focus == NULL || level > focus->value[0]) )
		{
		    weak_focus = TRUE;
		    mana = mana * 7 / 4;
		    sprintf( buf2, "%s*", skill_table[sn].name );
		}
		else
		    sprintf( buf2, "%s", skill_table[sn].name );

		sprintf( buf, "%-18.18s %3d%% %s %3d mana", 
                    buf2, 
                    ch->pcdata->learned[sn],
                    buf3, 
                    mana );
	    }
 
	    if ( spell_list[level][0] == '\0' )
		sprintf( spell_list[level], "\n\r%sLevel %2d:%s\n\r %s",
		    P_COLOR(ch, C_TITLES),
		    level, 
		    P_COLOR(ch, C_INFORMATION),
		    buf );
	    else /* append */
	    {
		if ( ++spell_columns[level] % 1 == 0 )
		    strcat( spell_list[level], "\n\r " );
		strcat( spell_list[level], buf );
	    }
	}
    }
 
    /* return results */
 
    if (!found)
    {
      	send_to_char("No spells found.\n\r",ch);
      	return;
    }

    buffer = new_buf();
    for (level = 0; level < LEVEL_HERO + 1; level++)
	if (spell_list[level][0] != '\0')
	    add_buf(buffer,spell_list[level]);
    add_buf(buffer,"\n\r");

    for ( sn = 0; sn < MAX_STYLES; sn++ )
    {
	if ( ch->pcdata->styles[sn]
	&&   IS_SET(style_table[sn].specialization, SPEC_MAGICAL) )
	{
	    sprintf( buf, "%sSpecialization:%s %s\n\r\n\r",
	        P_COLOR(ch, C_TITLES),
	        style_table[sn].name,
	        P_COLOR(ch, C_INFORMATION) );
	    add_buf( buffer, buf );
	    break;
	}
    }

    add_buf( buffer, P_COLOR(ch, C_TITLES) );
    if ( !fAll )
	add_buf( buffer, "\n\rUse 'spells all' for a full list." );
    if ( weak_focus )
	if ( focus != NULL )
	    add_buf( buffer, "\n\r* Focus inadequate for casting this spell." );
	else
	    add_buf( buffer, "\n\r* Focus required to lower mana cost." );
    add_buf( buffer, "\n\r" );
    add_buf( buffer, P_COLOR_OFF(ch) );

    page_to_char( buf_string(buffer), ch );
    free_buf( buffer );
}

void do_skills(CHAR_DATA *ch, char *argument)
{
    BUFFER *buffer;
    char arg[MAX_INPUT_LENGTH];
#if defined(macintosh)
    static char skill_list[LEVEL_HERO + 1][MAX_STRING_LENGTH];
#else
    char skill_list[LEVEL_HERO + 1][MAX_STRING_LENGTH];
#endif
    char skill_columns[LEVEL_HERO + 1];
    int sn, level, min_lev = 1, max_lev = LEVEL_HERO;
    bool fAll = FALSE, found = FALSE;
    char buf[MAX_STRING_LENGTH];
    bool first = TRUE;
    int col;
 
    if (IS_NPC(ch))
      return;

    if (argument[0] != '\0')
    {
	fAll = TRUE;

	if (str_prefix(argument,"all"))
	{
	    argument = one_argument(argument,arg);
	    if (!is_number(arg))
	    {
		send_to_char("Arguments must be numerical or all.\n\r",ch);
		return;
	    }
	    max_lev = atoi(arg);

	    if (max_lev < 1 || max_lev > LEVEL_HERO)
	    {
		sprintf(buf,"Levels must be between 1 and %d.\n\r",LEVEL_HERO);
		send_to_char(buf,ch);
		return;
	    }

	    if (argument[0] != '\0')
	    {
		argument = one_argument(argument,arg);
		if (!is_number(arg))
		{
		    send_to_char("Arguments must be numerical or all.\n\r",ch);
		    return;
		}
		min_lev = max_lev;
		max_lev = atoi(arg);

		if (max_lev < 1 || max_lev > LEVEL_HERO)
		{
		    sprintf(buf,
			"Levels must be between 1 and %d.\n\r",LEVEL_HERO);
		    send_to_char(buf,ch);
		    return;
		}

		if (min_lev > max_lev)
		{
		    send_to_char("That would be silly.\n\r",ch);
		    return;
		}
	    }
	}
    }

    /* initialize data */
    for (level = 0; level < LEVEL_HERO + 1; level++)
    {
        skill_columns[level] = 0;
        skill_list[level][0] = '\0';
    }
 
    for (sn = 0; sn < max_player_skill; sn++)
    {
        if (skill_table[sn].name == NULL )
	    break;

	if ((level = skill_level(ch, sn)) < LEVEL_HERO + 1
	&&  (fAll || level <= ch->level)
	&&  level >= min_lev && level <= max_lev
	&&  skill_table[sn].spell_fun == spell_null
	&&  ch->pcdata->learned[sn] > 0)
        {
	    found = TRUE;
/*	    level = skill_table[sn].skill_level[ch->class];*/
	    if (ch->level < level)
	    	sprintf(buf,"%-18.18s n/a           ", skill_table[sn].name);
	    else
            {
                char buf2[MAX_STRING_LENGTH];
                if ( ch->pcdata->skill_mods[sn] )
                    sprintf( buf2, "(%s%3d%%)",
                        ch->pcdata->skill_mods[sn] > 0 ? "+" : "-",
                        ch->pcdata->skill_mods[sn] > 0
                         ? ch->pcdata->skill_mods[sn]
                         : ch->pcdata->skill_mods[sn] * -1 );
                else
                    strcpy( buf2, "      " );
	    	sprintf( buf, "%-18.18s %3d%% %s   ",
                    skill_table[sn].name,
		    ch->pcdata->learned[sn],
                    buf2 );
            }
 
	    if (skill_list[level][0] == '\0')
          	sprintf( skill_list[level], "\n\r%sLevel %2d:%s %s",
          	    P_COLOR(ch, C_TITLES),
          	    level,
          	    P_COLOR(ch, C_INFORMATION),
          	    buf );
	    else /* append */
	    {
          	if ( ++skill_columns[level] % 2 == 0)
		    strcat(skill_list[level],"\n\r          ");
          	strcat(skill_list[level],buf);
	    }
	}
    }
 
    /* return results */
 
    if (!found)
    {
      	send_to_char("No skills found.\n\r",ch);
      	return;
    }

    buffer = new_buf();
    for (level = 0; level < LEVEL_HERO + 1; level++)
	if (skill_list[level][0] != '\0')
	    add_buf(buffer,skill_list[level]);
    add_buf(buffer,"\n\r");
    
    col = 0;
    for ( sn = 0; sn < MAX_STYLES; sn++ )
    {
	if ( ch->pcdata->styles[sn]
	&&   IS_SET(style_table[sn].specialization, SPEC_STYLE) )
	{
	    if ( first )
	    {
	    	sprintf( buf, "\n\r%sCombat styles:%s\n\r",
	    	    P_COLOR(ch, C_TITLES), P_COLOR(ch, C_INFORMATION) );
		add_buf( buffer, buf );
		first = FALSE;
	    }
	    
	    sprintf( buf, "%-25s ", style_table[sn].name );
	    if ( ++col % 3 == 0 )
		strcat( buf, "\n\r" );

	    add_buf( buffer, buf );
	}
    }
    if ( col % 3 != 0 )
	add_buf( buffer, "\n\r" );
    add_buf( buffer, "\n\r" );
    
    if ( !fAll )
    {
	sprintf( buf, "\n\r%sUse 'skills all' for a full list.%s\n\r",
	    P_COLOR(ch, C_TITLES), P_COLOR(ch, C_INFORMATION) );
	add_buf( buffer, buf );
    }

    add_buf( buffer, P_COLOR_OFF(ch) );

    page_to_char( buf_string(buffer), ch );
    free_buf( buffer );
}

/* shows skills, groups and costs (only if not bought) */
void list_group_costs(CHAR_DATA *ch)
{
    char buf[100];
    int gn,sn,col;

    if (IS_NPC(ch))
	return;

    col = 0;

    sprintf(buf,"%-18s %-5s %-18s %-5s %-18s %-5s\n\r","group","cp","group","cp","group","cp");
    send_to_char(buf,ch);

    for (gn = 0; gn < MAX_GROUP; gn++)
    {
	if (group_table[gn].name == NULL)
	    break;

        if (!ch->gen_data->group_chosen[gn] 
	&&  !ch->pcdata->group_known[gn]
	&&  group_table[gn].rating[ch->class] > 0
	&&  group_table[gn].gainable
	&&  group_usable(ch, gn) )
	{
	    sprintf(buf,"%-18s %-5d ",group_table[gn].name,
				    group_table[gn].rating[ch->class]);
	    send_to_char(buf,ch);
	    if (++col % 3 == 0)
		send_to_char("\n\r",ch);
	}
    }
    if ( col % 3 != 0 )
        send_to_char( "\n\r", ch );
    send_to_char("\n\r",ch);

    col = 0;
 
    sprintf(buf,"%-18s %-5s %-18s %-5s %-18s %-5s\n\r","skill","cp","skill","cp","skill","cp");
    send_to_char(buf,ch);
 
    for (sn = 0; sn < max_player_skill; sn++)
    {
        if (skill_table[sn].name == NULL)
            break;
 
        if (!ch->gen_data->skill_chosen[sn] 
	&&  ch->pcdata->learned[sn] == 0
	&&  skill_table[sn].spell_fun == spell_null
	&&  skill_rating(ch, sn) > 0
	&&  skill_table[sn].gainable
	&&  skill_usable(ch, sn) )
        {
            sprintf(buf,"%-18s %-5d ",skill_table[sn].name,
                                    skill_rating(ch, sn));
            send_to_char(buf,ch);
            if (++col % 3 == 0)
                send_to_char("\n\r",ch);
        }
    }
    if ( col % 3 != 0 )
        send_to_char( "\n\r", ch );
    send_to_char( "\n\r", ch );

    col = 0;
        
    sprintf( buf, "%-31s%-4s   %-31s%-4s\n\r", 
        "style/specialization", "cost",
        "style/specialization", "cost" );
        
    send_to_char( buf, ch );

    for ( sn = 0; sn < MAX_STYLES; sn++ )
    {
	if ( style_table[sn].name == NULL )
	    break;

	if ( !is_specialized(ch, sn)
	&&   style_table[sn].rating[ch->class] > 0 )
	{
	    sprintf( buf, "%-31s%-2d     ",
	        style_table[sn].name, style_table[sn].rating[ch->class] );
	    send_to_char( buf, ch );
	    if ( ++col % 2 == 0 )
		send_to_char( "\n\r", ch ); 
	}
    }
	
    if ( col % 2 != 0 )
	send_to_char( "\n\r", ch );

    send_to_char("\n\r",ch);

    sprintf(buf,"Creation points: %d\n\r",ch->pcdata->points);
    send_to_char(buf,ch);
    sprintf(buf,"Experience per level: %d\n\r",
	    exp_per_level(ch,ch->gen_data->points_chosen));
    send_to_char(buf,ch);
    return;
}


void list_group_chosen(CHAR_DATA *ch)
{
    char buf[100];
    int gn,sn,col;
 
    if (IS_NPC(ch))
        return;
 
    col = 0;
 
    sprintf(buf,"%-18s %-5s %-18s %-5s %-18s %-5s","group","cp","group","cp","group","cp\n\r");
    send_to_char(buf,ch);
 
    for (gn = 0; gn < MAX_GROUP; gn++)
    {
        if (group_table[gn].name == NULL)
            break;
 
        if (ch->gen_data->group_chosen[gn] 
	&&  group_table[gn].rating[ch->class] > 0)
        {
            sprintf(buf,"%-18s %-5d ",group_table[gn].name,
                                    group_table[gn].rating[ch->class]);
            send_to_char(buf,ch);
            if (++col % 3 == 0)
                send_to_char("\n\r",ch);
        }
    }
    if ( col % 3 != 0 )
        send_to_char( "\n\r", ch );
    send_to_char("\n\r",ch);
 
    col = 0;
 
    sprintf(buf,"%-18s %-5s %-18s %-5s %-18s %-5s","skill","cp","skill","cp","skill","cp\n\r");
    send_to_char(buf,ch);
 
    for (sn = 0; sn < max_player_skill; sn++)
    {
        if (skill_table[sn].name == NULL)
            break;
 
        if (ch->gen_data->skill_chosen[sn] 
	&&  skill_rating(ch, sn) > 0)
        {
            sprintf(buf,"%-18s %-5d ", skill_table[sn].name,
                                       skill_rating(ch, sn));
            send_to_char(buf,ch);
            if (++col % 3 == 0)
                send_to_char("\n\r",ch);
        }
    }
    if ( col % 3 != 0 )
        send_to_char( "\n\r", ch );
    send_to_char("\n\r",ch);

    col = 0;
        
    sprintf( buf, "%-31s%-4s   %-31s%-4s\n\r", 
        "style/specialization", "cost",
        "style/specialization", "cost" );
        
    send_to_char( buf, ch );

    for ( sn = 0; sn < MAX_STYLES; sn++ )
    {
	if ( style_table[sn].name == NULL )
	    break;

	if ( ch->gen_data->skill_chosen[sn]
	&&   style_table[sn].rating[ch->class] > 0 )
	{
	    sprintf( buf, "%-31s%-2d     ",
	        style_table[sn].name, style_table[sn].rating[ch->class] );
	    send_to_char( buf, ch );
	    if ( ++col % 2 == 0 )
		send_to_char( "\n\r", ch ); 
	}
    }
	
    if ( col % 2 != 0 )
	send_to_char( "\n\r", ch );
    send_to_char("\n\r",ch);

    sprintf(buf,"Creation points: %d\n\r",ch->gen_data->points_chosen);
    send_to_char(buf,ch);
    sprintf(buf,"Experience per level: %d\n\r",
	    exp_per_level(ch,ch->gen_data->points_chosen));
    send_to_char(buf,ch);
    return;
}

int exp_per_level(CHAR_DATA *ch, int points)
{
    int expl,inc,explvl;

    if (IS_NPC(ch))
	return 1000; 

    expl = 1000;
    inc = 500;
    explvl = 0;

    if (points < 40)
	explvl = (1000 * (pc_race_table[ch->race].class_mult[ch->class] ?
		       pc_race_table[ch->race].class_mult[ch->class]/100 : 1));

    if ((points < 40) && (explvl == 0))
	return 1000;
    else if (points < 40)
	return explvl;

    /* processing */
    points -= 40;

    while (points > 9)
    {
	expl += inc;
        points -= 10;
        if (points > 9)
	{
	    expl += inc;
	    inc *= 2;
	    points -= 10;
	}
    }

    expl += points * inc / 10;  

    return expl * pc_race_table[ch->race].class_mult[ch->class]/100;
}

/* this procedure handles the input parsing for the skill generator */
bool parse_gen_groups(CHAR_DATA *ch,char *argument)
{
    char arg[MAX_INPUT_LENGTH];
    char buf[100], buf2[MAX_STRING_LENGTH];
    int gn,sn,i;
 
    if (argument[0] == '\0')
	return FALSE;

    argument = one_argument(argument,arg);

    if (!str_prefix(arg,"help"))
    {
	if (argument[0] == '\0')
	{
	    do_help(ch,"group help");
	    return TRUE;
	}

        do_help(ch,argument);
	return TRUE;
    }

    if (!str_prefix(arg,"add"))
    {
	if (argument[0] == '\0')
	{
	    send_to_char("You must provide a skill name.\n\r",ch);
	    return TRUE;
	}

	gn = group_lookup(argument);
	if (gn != -1)
	{
	    if (ch->gen_data->group_chosen[gn]
	    ||  ch->pcdata->group_known[gn])
	    {
		send_to_char("You already know that group!\n\r",ch);
		return TRUE;
	    }

	    if (group_table[gn].rating[ch->class] < 1
	    ||  !group_table[gn].gainable
	    ||  !group_usable(ch, gn) )
	    {
	  	send_to_char("That group is not available.\n\r",ch);
	 	return TRUE;
	    }

           /* Close security hole */
            if ( ch->gen_data->points_chosen + group_table[gn].rating[ch->class] > MAX_POINTS )
            {
                send_to_char(
                    "Your head cannot contain that much knowledge.\n\r", ch);
                return TRUE;
            }

	    sprintf(buf,"%s group added\n\r",group_table[gn].name);
	    send_to_char(buf,ch);
	    ch->gen_data->group_chosen[gn] = TRUE;
	    ch->gen_data->points_chosen += group_table[gn].rating[ch->class];
	    gn_add(ch,gn);
	    ch->pcdata->points += group_table[gn].rating[ch->class];
	    return TRUE;
	}

	sn = skill_lookup(argument);
	if (sn != -1)
	{
	    if (ch->gen_data->skill_chosen[sn]
	    ||  ch->pcdata->learned[sn] > 0)
	    {
		send_to_char("You already know that skill!\n\r",ch);
		return TRUE;
	    }

	    if (skill_rating(ch, sn) < 1
	    ||  skill_table[sn].spell_fun != spell_null
	    ||  !skill_usable(ch, sn)
	    ||  !skill_table[sn].gainable )
	    {
		send_to_char("That skill is not available.\n\r",ch);
		return TRUE;
	    }

           /* Close security hole */
            if (ch->gen_data->points_chosen + skill_table[sn].rating[ch->class] > MAX_POINTS )
            {
                send_to_char(
                    "Your head cannot contain that much knowledge.\n\r", ch);
                return TRUE;
            }

	    sprintf(buf, "%s skill added\n\r",skill_table[sn].name);
	    send_to_char(buf,ch);
	    ch->gen_data->skill_chosen[sn] = TRUE;
	    ch->gen_data->points_chosen += skill_rating(ch, sn);
	    ch->pcdata->learned[sn] = 1;
	    ch->pcdata->points += skill_rating(ch, sn);
	    return TRUE;
	}

	sn = style_lookup( argument );
	if ( sn != -1 )
	{
	    if ( ch->gen_data->style_chosen[sn]
	    ||   ch->pcdata->styles[sn] )
	    {
		send_to_char( "You already know that!\n\r", ch );
		return TRUE;
	    }

	    if ( style_table[sn].rating[ch->class] <= 0 )
	    {
		send_to_char( "That isn't available.\n\r",ch);
		return TRUE;
	    }
	    
            if ( !IS_NULLSTR(style_table[sn].prereq)
            &&   !ch->pcdata->learned[skill_lookup(style_table[sn].prereq)] )
            {
        	sprintf( buf2, "You need to learn %s first.", style_table[sn].prereq );
                send_to_char( buf2, ch );
		return TRUE;
            }

	    sprintf( buf, "%s added\n\r", style_table[sn].name );
	    send_to_char( buf, ch );
	    ch->gen_data->style_chosen[sn] = TRUE;
	    ch->gen_data->points_chosen += style_table[sn].rating[ch->class];
	    ch->pcdata->styles[sn] = TRUE;
	    ch->pcdata->points += style_table[sn].rating[ch->class];
	    return TRUE;
	}
	
	send_to_char("No skills or groups by that name...\n\r",ch);
	return TRUE;
    }

    if (!strcmp(arg,"drop"))
    {
	if (argument[0] == '\0')
  	{
	    send_to_char("You must provide a skill to drop.\n\r",ch);
	    return TRUE;
	}

	gn = group_lookup(argument);
	if (gn != -1 && ch->gen_data->group_chosen[gn])
	{
	    send_to_char("Group dropped.\n\r",ch);
	    ch->gen_data->group_chosen[gn] = FALSE;
	    ch->gen_data->points_chosen -= group_table[gn].rating[ch->class];
	    gn_remove(ch,gn);
	    for (i = 0; i < MAX_GROUP; i++)
	    {
		if (ch->gen_data->group_chosen[gn])
		    gn_add(ch,gn);
	    }
	    ch->pcdata->points -= group_table[gn].rating[ch->class];
	    return TRUE;
	}

	sn = skill_lookup(argument);
	if (sn != -1 && ch->gen_data->skill_chosen[sn])
	{
	    send_to_char("Skill dropped.\n\r",ch);
	    ch->gen_data->skill_chosen[sn] = FALSE;
	    ch->gen_data->points_chosen -= skill_rating(ch, sn);
	    ch->pcdata->learned[sn] = 0;
	    ch->pcdata->points -= skill_rating(ch, sn);
	    return TRUE;
	}

	sn = style_lookup( argument );
	if ( sn != -1 && ch->gen_data->style_chosen[sn] )
	{
	    send_to_char( "Dropped.\n\r", ch );
	    ch->gen_data->style_chosen[sn] = FALSE;
	    ch->gen_data->points_chosen -= style_table[sn].rating[ch->class];
	    ch->pcdata->styles[sn] = FALSE;
	    ch->pcdata->points -= style_table[sn].rating[ch->class];
	    return TRUE;
	}
	
	send_to_char("You haven't bought any such skill or group.\n\r",ch);
	return TRUE;
    }

    if (!str_prefix(arg,"premise"))
    {
	do_help(ch,"premise");
	return TRUE;
    }

    if (!str_prefix(arg,"list"))
    {
	list_group_costs(ch);
	return TRUE;
    }

    if (!str_prefix(arg,"learned"))
    {
	list_group_chosen(ch);
	return TRUE;
    }

    if (!str_prefix(arg,"info"))
    {
	do_groups(ch,argument);
	return TRUE;
    }

    return FALSE;
}
	    
	


        

/* shows all groups, or the sub-members of a group */
void do_groups(CHAR_DATA *ch, char *argument)
{
    char buf[100];
    int gn,sn,col;

    if (IS_NPC(ch))
	return;

    col = 0;

    if (argument[0] == '\0')
    {   /* show all groups */
	
	for (gn = 0; gn < MAX_GROUP; gn++)
        {
	    if (group_table[gn].name == NULL)
		break;
	    if (ch->pcdata->group_known[gn])
	    {
		sprintf(buf,"%-20s ",group_table[gn].name);
		send_to_char(buf,ch);
		if (++col % 3 == 0)
		    send_to_char("\n\r",ch);
	    }
        }
        if ( col % 3 != 0 )
            send_to_char( "\n\r", ch );
        sprintf(buf,"Creation points: %d\n\r",ch->pcdata->points);
	send_to_char(buf,ch);
	return;
     }

     if (!str_cmp(argument,"all"))    /* show all groups */
     {
        for (gn = 0; gn < MAX_GROUP; gn++)
        {
            if (group_table[gn].name == NULL)
                break;
	    sprintf(buf,"%-20s ",group_table[gn].name);
            send_to_char(buf,ch);
	    if (++col % 3 == 0)
            	send_to_char("\n\r",ch);
        }
        if ( col % 3 != 0 )
            send_to_char( "\n\r", ch );
	return;
     }
     
     /* show the sub-members of a group */
     gn = group_lookup( argument );
     if (gn == -1)
     {
	send_to_char("No group of that name exist.\n\r",ch);
	send_to_char(
	    "Type 'groups all' or 'info all' for a full listing.\n\r",ch);
	return;
     }

     for ( sn = 0; sn < MAX_IN_GROUP; sn++ )
     {
	int spell;
	
	if (group_table[gn].spells[sn] == NULL)
	    break;
	
	spell = skill_lookup( group_table[gn].spells[sn] );
	if ( spell != -1 
	&&   skill_usable(ch, spell) 
	&&   LEVEL_HERO >= skill_level(ch, spell) )
	    sprintf(buf, "%-20s %2d   ",
                group_table[gn].spells[sn],
                skill_level(ch, spell) );
	else
	{
	    int group = spell == -1 ?
                        group_lookup( group_table[gn].spells[sn] ) : -1;
	    if ( group != -1
	    &&   group_usable(ch, group) 
	    &&   group_table[group].gainable )
		sprintf(buf,"%-20s      ",group_table[gn].spells[sn]);
	    else
	    {
		char buf2[22];
		sprintf(buf2, "%s*", group_table[gn].spells[sn]);
		sprintf(buf, "%-20s      ", buf2);
	    }
	}

	send_to_char( buf, ch );
	if (++col % 3 == 0)
	    send_to_char("\n\r",ch);
     }
    if ( col % 3 != 0 )
        send_to_char( "\n\r", ch );
    
    send_to_char( "* indicates unavailable to this class.\n\r", ch );
}

/* checks for skill improvement */
void check_improve( CHAR_DATA *ch, int sn, bool success, int multiplier, bool msg_vis )
{
    int chance;
    char buf[100];

    if (IS_NPC(ch))
	return;

    if (ch->level < skill_level(ch, sn)
    ||  skill_rating(ch, sn) == 0
    ||  ch->pcdata->learned[sn] == 0
    ||  ch->pcdata->learned[sn] == 100)
	return;  /* skill is not known */ 

    /* check to see if the character has a chance to learn */
    chance = 10 * int_app[get_curr_stat(ch,STAT_INT)/4].learn;
    chance /= ( multiplier * skill_rating(ch, sn) * 4 );
    chance += ch->level;

    if (number_range(1,1000) > chance)
	return;

    /* now that the character has a CHANCE to learn, see if they really have */	

    if (success)
    {
	chance = URANGE(5,100 - ch->pcdata->learned[sn], 95);
	if (number_percent() < chance)
	{
	    if ( msg_vis )
	    {
	    sprintf(buf,"%sYou have become better at %s!%s\n\r",
		P_COLOR_LITERAL(ch, COLOR_WHITE),
		skill_table[sn].name,
		P_COLOR_OFF(ch) );
	    send_to_char(buf,ch);
	    }
	    ch->pcdata->learned[sn]++;
	    gain_exp(ch,2 * skill_rating(ch, sn));
	}
    }

    else
    {
	chance = URANGE(5,ch->pcdata->learned[sn]/2,30);
	if (number_percent() < chance)
	{
	    if ( msg_vis )
	    {
	    sprintf(buf,
		"%sYou learn from your mistakes, and your %s skill improves!%s\n\r",
		P_COLOR_LITERAL(ch, COLOR_WHITE),
		skill_table[sn].name,
		P_COLOR_OFF(ch) );
	    send_to_char(buf,ch);
	    }
	    ch->pcdata->learned[sn] += number_range(1,3);
	    ch->pcdata->learned[sn] = UMIN(ch->pcdata->learned[sn],100);
	    gain_exp(ch,2 * skill_rating(ch, sn));
	}
    }
}

/* returns a group index number given the name */
int group_lookup( const char *name )
{
    int partial = -1;
    int gn;

    if (name == "maladictions")
	name = "maledictions";

    for ( gn = 0; gn < MAX_GROUP; gn++ )
    {
        if ( group_table[gn].name == NULL )
            break;

        if ( LOWER(name[0]) == LOWER(group_table[gn].name[0]) )
	{
	    if ( !str_cmp( name, group_table[gn].name ) )
		return gn;

            if ( !str_prefix( name, group_table[gn].name )
	    &&   partial == -1 )
	    {
		partial = gn;
		continue;
	    }
	}
    }
 
    return partial;
}

/* returns a group index number given the name */
/*
 *int group_lookup_abs( const char *name )
 *{
 *    int gn;
 * 
 *    for ( gn = 0; gn < MAX_GROUP; gn++ )
 *    {
 *        if ( group_table[gn].name == NULL )
 *            break;
 *        if ( LOWER(name[0]) == LOWER(group_table[gn].name[0])
 *        &&   !str_cmp( name, group_table[gn].name ) )
 *            return gn;
 *    }
 * 
 *    return -1;
 *}
 */

/* recursively adds a group given its number -- uses group_add */
void gn_add( CHAR_DATA *ch, int gn)
{
    int i;
    
    ch->pcdata->group_known[gn] = TRUE;
    for ( i = 0; i < MAX_IN_GROUP; i++)
    {
        if (group_table[gn].spells[i] == NULL)
            break;
        group_add(ch,group_table[gn].spells[i],FALSE);
    }
}

/* recusively removes a group given its number -- uses group_remove */
void gn_remove( CHAR_DATA *ch, int gn)
{
    int i;

    ch->pcdata->group_known[gn] = FALSE;

    for ( i = 0; i < MAX_IN_GROUP; i ++)
    {
	if (group_table[gn].spells[i] == NULL)
	    break;
	group_remove(ch,group_table[gn].spells[i]);
    }
}
	
/* use for processing a skill or group for addition  */
void group_add( CHAR_DATA *ch, const char *name, bool deduct)
{
    int sn,gn;

    if (IS_NPC(ch)) /* NPCs do not have skills */
	return;

    sn = skill_lookup_abs(name);

    if (sn != -1)
    {
	if (ch->pcdata->learned[sn] == 0) /* i.e. not known */
	{
            if ( skill_usable(ch, sn) ) 
            {
	        ch->pcdata->learned[sn] = 1;
	        if (deduct)
	   	        ch->pcdata->points += skill_rating(ch, sn);
	    } 
	}
	return;
    }
	
    /* now check groups */

    gn = group_lookup(name);

    if (gn != -1)
    {
	if (ch->pcdata->group_known[gn] == FALSE)  
	{
            if ( group_usable(ch, gn) ) {
	        ch->pcdata->group_known[gn] = TRUE;
	        if (deduct)
		    ch->pcdata->points += group_table[gn].rating[ch->class];
            }
	}
	gn_add(ch,gn); /* make sure all skills in the group are known */
    }
}

/* used for processing a skill or group for deletion -- no points back! */

void group_remove(CHAR_DATA *ch, const char *name)
{
    int sn, gn;
    
     sn = skill_lookup(name);

    if (sn != -1)
    {
	ch->pcdata->learned[sn] = 0;
	return;
    }
 
    /* now check groups */
 
    gn = group_lookup(name);
 
    if (gn != -1 && ch->pcdata->group_known[gn] == TRUE)
    {
	ch->pcdata->group_known[gn] = FALSE;
	gn_remove(ch,gn);  /* be sure to call gn_add on all remaining groups */
    }
}


/* Can the given group be used by this character? */
bool group_usable( CHAR_DATA *ch, int gn )
{
    int i;
    bool empty = TRUE;
    int sclass = group_table[gn].sclass;

    if (sclass != SCLASS_NONE 
    &&  religion_table[GET_GOD(ch)].skill_modifications[sclass].unavailable )
        return FALSE;


    if ( ch->hometown > 0
    &&   !IS_NULLSTR(group_table[gn].unhometowns)
    &&   is_name(hometown_table[ch->hometown].ht_name, group_table[gn].unhometowns) )
        return FALSE;

    if ( ch->hometown > 0
    &&   !IS_NULLSTR(group_table[gn].hometowns)
    &&   !is_name(hometown_table[ch->hometown].ht_name, group_table[gn].hometowns) )
        return FALSE;


    for ( i = 0; i < MAX_IN_GROUP; i++ )
    {
        int rgn, rsn;

        if ( group_table[gn].spells[i] == NULL
	||   group_table[gn].spells[i][0] == '\0' )
            break;

	empty = FALSE;

        rsn = skill_lookup( group_table[gn].spells[i] );
        if ( rsn != -1 ) {
            if ( skill_usable(ch, rsn)
	    &&    LEVEL_HERO >= skill_level(ch, rsn) )
                return TRUE;
                
            continue;
        }

        rgn = group_lookup( group_table[gn].spells[i] );
        if ( rgn != -1 )
        {
            if ( group_usable(ch, rgn) )
                return TRUE;
        }

    }

    return empty;
}


/* Can the given skill be used by this character? */
bool skill_usable( CHAR_DATA *ch, int sn )
{
    int sclass = skill_table[sn].sclass;

    if ( religion_table[GET_GOD(ch)].skill_modifications[sclass].unavailable )
        return FALSE;

    if ( class_table[ch->class].pMana >= PURE_MAGIC ) {
        if (religion_table[GET_GOD(ch)].skill_modifications[sclass].caster.unavail_subs
        &   skill_table[sn].sub_class )
            return FALSE;
    } else {
        if (religion_table[GET_GOD(ch)].skill_modifications[sclass].non_caster.unavail_subs
        &   skill_table[sn].sub_class )
            return FALSE;
    }
    
    if ( (skill_table[sn].required_form
    &&    (skill_table[sn].required_form & ch->form) == 0)
    ||   (skill_table[sn].disallowed_form & ch->form) != 0
    ||   (skill_table[sn].required_parts
    &&    (skill_table[sn].required_parts & ch->parts) == 0)
    ||   (skill_table[sn].disallowed_parts & ch->parts) != 0 )
	return FALSE;

    if ( ch->hometown > 0
    &&   !IS_NULLSTR(skill_table[sn].unhometowns)
    &&   is_name(hometown_table[ch->hometown].ht_name, skill_table[sn].unhometowns) )
        return FALSE;

    if ( ch->hometown > 0
    &&   !IS_NULLSTR(skill_table[sn].hometowns)
    &&   !is_name(hometown_table[ch->hometown].ht_name, skill_table[sn].hometowns) )
        return FALSE;

    return TRUE;
}


/* A characters effective level at a skill. */
int effective_level( CHAR_DATA *ch, int sn )
{
    bool caster, non_caster;
    int change, level, i;
        
    if ( IS_NPC(ch) )
	return ch->level;

    if ( sn == -1 )   /* Covers exotic weapons */
	return ch->level;

    caster = non_caster = FALSE;
    if ( class_table[ch->class].pMana >= PURE_MAGIC )
	caster		= TRUE;
    else
	non_caster	= TRUE;

    for ( i = 0; i < MAX_PROFESSIONS - 1; i++ )
    {
	if ( ch->pcdata->former_professions[i] == -1 )
	    break;
	
	if ( class_table[ch->pcdata->former_professions[i]].pMana >= PURE_MAGIC )
	    caster	= TRUE;
	else
	    non_caster	= TRUE;
    }

    if ( caster && non_caster )
	change = UMAX(
	    religion_table[GET_GOD(ch)].skill_modifications[skill_table[sn].sclass].caster.level_mod,
	    religion_table[GET_GOD(ch)].skill_modifications[skill_table[sn].sclass].non_caster.level_mod );
    else if ( caster )
	change = religion_table[GET_GOD(ch)].skill_modifications[skill_table[sn].sclass].caster.level_mod;
    else
	change = religion_table[GET_GOD(ch)].skill_modifications[skill_table[sn].sclass].non_caster.level_mod;
	
    level = (ch->level * (100 + change)) / 100;
    
    if ( skill_table[sn].spell_fun != NULL )
    	return level;
    
    if ( IS_SET(class_table[ch->class].flags, CLASS_OUTDOORS)
    &&   ch->in_room != NULL
    &&   (ch->in_room->sector_type == SECT_FIELD
    ||    ch->in_room->sector_type == SECT_FOREST
    ||    ch->in_room->sector_type == SECT_HILLS
    ||    ch->in_room->sector_type == SECT_MOUNTAIN
    ||    ch->in_room->sector_type == SECT_DESERT) )
	level += 2;
    
    if ( ch->level < LEVEL_HERO / 2 )
	return level;
    
    switch ( SEASON ) {
    case SEASON_WINTER:
	if ( IS_SET(religion_table[GET_GOD(ch)].flags, GOD_WINTER) )
	    level++;
	else if ( IS_SET(religion_table[GET_GOD(ch)].flags, GOD_SUMMER) )
	    level--;
	break;
    case SEASON_SPRING:
	if ( IS_SET(religion_table[GET_GOD(ch)].flags, GOD_SPRING) )
	    level++;
	else if ( IS_SET(religion_table[GET_GOD(ch)].flags, GOD_FALL) )
	    level--;
	break;
    case SEASON_SUMMER:
	if ( IS_SET(religion_table[GET_GOD(ch)].flags, GOD_SUMMER) )
	    level++;
	else if ( IS_SET(religion_table[GET_GOD(ch)].flags, GOD_WINTER) )
	    level--;
	break;
    case SEASON_FALL:
	if ( IS_SET(religion_table[GET_GOD(ch)].flags, GOD_FALL) )
	    level++;
	else if ( IS_SET(religion_table[GET_GOD(ch)].flags, GOD_SPRING) )
	    level--;
	break;
    }
    
    switch ( LUNAR_PHASE )
    {
    case MOON_NEW:
	if (IS_GOOD(ch))
	    level += 1;
	else if (IS_EVIL(ch))
	    level -= 1;
    case MOON_FULL:
	if (IS_GOOD(ch))
	    level -= 1;
	else if (IS_EVIL(ch))
	    level += 1;
    }
    
    return UMAX(1, level);
}


int skill_level( CHAR_DATA *ch, int sn )
{
    int i, level;
    
    if ( IS_NPC(ch) )
	return 1;
	
    level = skill_table[sn].skill_level[ch->class];
    
    for ( i = 0; i < MAX_PROFESSIONS - 1; i++ )
    {
	if ( ch->pcdata->former_professions[i] == -1 )
	    break;

	level = UMIN( level, 
	    skill_table[sn].skill_level[ch->pcdata->former_professions[i]] );
    }
    
    return level;
}


int skill_rating( CHAR_DATA *ch, int sn )
{
    int i, rating;
    
    if ( IS_NPC(ch) )
	return 0;

    rating = skill_table[sn].rating[ch->class];

    for ( i = 0; i < MAX_PROFESSIONS - 1; i++ )
    {
	if ( ch->pcdata->former_professions[i] == -1 )
	    break;

	if ( rating == 0 )
	    rating = skill_table[sn].rating[ch->pcdata->former_professions[i]];
	else if ( skill_table[sn].rating[ch->pcdata->former_professions[i]] > 0 )
	    rating = UMIN( rating,
	        skill_table[sn].rating[ch->pcdata->former_professions[i]] );
    }
    
    return rating;
}


bool is_specialized( CHAR_DATA *ch, int sn )
{
    int i;

    if ( IS_NPC(ch)
    ||   (IS_SET(style_table[sn].specialization, SPEC_STYLE)
    &&    ch->pcdata->styles[sn]) )
	return TRUE;

    if ( IS_SET(style_table[sn].specialization, SPEC_NO_REMORT)
    &&   ch->pcdata->former_professions[0] != -1 )
	return TRUE;
	
    if ( IS_SET(style_table[sn].specialization, SPEC_STYLE) )
	return FALSE;

    for ( i=0; i<MAX_STYLES; i++ )
    {
	if ( i == sn )
	    continue;
	    
	if ( (style_table[sn].specialization & !SPEC_NO_REMORT) & style_table[i].specialization
	&&   ch->pcdata->styles[i] )
	    return TRUE;
    }
    
    return FALSE;
}


bool has_specialization( CHAR_DATA *ch, int sn )
{
    if ( IS_NPC(ch) )
	return FALSE;

    return ch->pcdata->styles[sn];
}

